//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "THasher.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
// class THashDigest
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
void __fastcall THashDigest::AssignValue(const byte* AValue, int ASize)
{
	byte* newValue = NULL;
	try {
		newValue = new byte[ASize];
		CopyMemory(newValue, AValue, ASize);
	}
	catch(...) {
		delete[] newValue;
		throw;
	}
	delete[] FValue;
	FValue = newValue;
	FSize = ASize;
}
//---------------------------------------------------------------------------
const byte* __fastcall THashDigest::GetValue() const
{
	return FValue;
}
//---------------------------------------------------------------------------
AnsiString __fastcall THashDigest::GetAsText() const
{
	AnsiString text;
	char* buffer = NULL;
	try {
		int textLength = Size * 2;
		buffer = new char[textLength + 1];
		BinToHex(reinterpret_cast<char*>(const_cast<byte*>(Value)), buffer,
			Size);
		text = AnsiString(buffer, textLength);
	}
	__finally {
		delete[] buffer;
	}
	return text;
}
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall THashDigest::THashDigest(const byte* AValue, int ASize) :
	FValue(NULL),
	FSize(0)
{
	AssignValue(AValue, ASize);
}
//---------------------------------------------------------------------------
__fastcall THashDigest::THashDigest(const THashDigest& ASrc) :
	FValue(NULL),
	FSize(0)
{
	*this = ASrc;
}
//---------------------------------------------------------------------------
__fastcall THashDigest::~THashDigest()
{
	delete[] FValue;
}
//---------------------------------------------------------------------------
THashDigest& __fastcall THashDigest::operator=(const THashDigest& ASrc)
{
	if(&ASrc != this) {
		AssignValue(ASrc.FValue, ASrc.FSize);
	}
	return *this;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// class THasher
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall THasher::THasher()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall THasher::~THasher()
{
	// Do nothing
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// class TMd5Hasher : public THasher
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TMd5Hasher::TMd5Hasher() :
	THasher(),
	FContext()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TMd5Hasher::~TMd5Hasher()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TMd5Hasher::Start()
{
	md5_starts(&FContext);
}
//---------------------------------------------------------------------------
void __fastcall TMd5Hasher::Update(const void* AData, int ALength)
{
	md5_update(&FContext,
		reinterpret_cast<uchar*>(const_cast<void*>(AData)), ALength);
}
//---------------------------------------------------------------------------
THashDigest __fastcall TMd5Hasher::Finish()
{
	uchar digest[16];
	md5_finish(&FContext, digest);
	return THashDigest(digest, sizeof(digest));
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// class TSha1Hasher : public THasher
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TSha1Hasher::TSha1Hasher() :
	THasher(),
	FContext()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TSha1Hasher::~TSha1Hasher()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TSha1Hasher::Start()
{
	sha1_starts(&FContext);
}
//---------------------------------------------------------------------------
void __fastcall TSha1Hasher::Update(const void* AData, int ALength)
{
	sha1_update(&FContext,
		reinterpret_cast<uchar*>(const_cast<void*>(AData)), ALength);
}
//---------------------------------------------------------------------------
THashDigest __fastcall TSha1Hasher::Finish()
{
	uchar digest[20];
	sha1_finish(&FContext, digest);
	return THashDigest(digest, sizeof(digest));
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// class TSha256Hasher : public THasher
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TSha256Hasher::TSha256Hasher() :
	THasher(),
	FContext()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TSha256Hasher::~TSha256Hasher()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TSha256Hasher::Start()
{
	sha2_starts(&FContext);
}
//---------------------------------------------------------------------------
void __fastcall TSha256Hasher::Update(const void* AData, int ALength)
{
	sha2_update(&FContext,
		reinterpret_cast<uchar*>(const_cast<void*>(AData)), ALength);
}
//---------------------------------------------------------------------------
THashDigest __fastcall TSha256Hasher::Finish()
{
	uchar digest[32];
	sha2_finish(&FContext, digest);
	return THashDigest(digest, sizeof(digest));
}
//---------------------------------------------------------------------------

