//---------------------------------------------------------------------------
#ifndef THasherH
#define THasherH
#include <Classes.hpp>
#include "md5.h"
#include "sha1.h"
#include "sha2.h"
//---------------------------------------------------------------------------
class THashDigest
{
private:
	byte* FValue;
	int FSize;
	void __fastcall AssignValue(const byte* AValue, int ASize);
	const byte* __fastcall GetValue() const;
	AnsiString __fastcall GetAsText() const;
protected:
public:
	__fastcall THashDigest(const byte* AValue, int ASize);
	__fastcall THashDigest(const THashDigest& ASrc);
	virtual __fastcall ~THashDigest();
	THashDigest& __fastcall operator=(const THashDigest& ASrc);
	__property const byte* Value = {read=GetValue};
	__property int Size = {read=FSize};
	__property AnsiString AsText = {read=GetAsText};
};
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
class THasher
{
private:
	__fastcall THasher(const THasher& ASrc);
	THasher& __fastcall operator=(const THasher& ASrc);
protected:
public:
	__fastcall THasher();
	virtual __fastcall ~THasher();
	virtual void __fastcall Start() = 0;
	virtual void __fastcall Update(const void* AData, int ALength) = 0;
	virtual THashDigest __fastcall Finish() = 0;
};
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
class TMd5Hasher : public THasher
{
private:
	md5_context FContext;
protected:
public:
	__fastcall TMd5Hasher();
	virtual __fastcall ~TMd5Hasher();
	virtual void __fastcall Start();
	virtual void __fastcall Update(const void* AData, int ALength);
	virtual THashDigest __fastcall Finish();
};
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
class TSha1Hasher : public THasher
{
private:
	sha1_context FContext;
protected:
public:
	__fastcall TSha1Hasher();
	virtual __fastcall ~TSha1Hasher();
	virtual void __fastcall Start();
	virtual void __fastcall Update(const void* AData, int ALength);
	virtual THashDigest __fastcall Finish();
};
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
class TSha256Hasher : public THasher
{
private:
	sha2_context FContext;
protected:
public:
	__fastcall TSha256Hasher();
	virtual __fastcall ~TSha256Hasher();
	virtual void __fastcall Start();
	virtual void __fastcall Update(const void* AData, int ALength);
	virtual THashDigest __fastcall Finish();
};
//---------------------------------------------------------------------------
#endif

