//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <clipbrd.hpp>
#include <strutils.hpp>
#include "TMainForm.h"
#include "TAutoCursor.h"
#include "TClipboardViewer.h"
#include "TFileExecutor.h"
#include "TFileNameList.h"
#include "TFileOperator.h"
#include "TListItemFinder.h"
#include "TMessageDialog.h"
#include "TProgressDialog.h"
#include "TScanThread.h"
#include "TShortCutFile.h"
#include "TTextFindDialog.h"
#include "SystemUtils.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMainForm* MainForm = NULL;
//---------------------------------------------------------------------------
// class TMainForm : public TForm
//---------------------------------------------------------------------------
// __published:
//---------------------------------------------------------------------------
void __fastcall TMainForm::ApplicationIdle(TObject* ASender, bool& ADone)
{
	ADone = true;
	Application->OnIdle = NULL;

	TAutoCursor busyNotification(crHourGlass);

	ProcessCommandParams();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::MainFormCloseQuery(TObject* ASender,
	bool& ACanClose)
{
	NewM1->Click();
	SaveSettings();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::NewM1Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	for(int i = 0; i < ListViews->Count; i++) {
		ListViews->Items[ListViews->Count - 1 - i]->Records->Clear();
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AddM1Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	TFileNameList* fileNames = NULL;
	try {
		fileNames = new TFileNameList();
		if(ASender == AddFilesM1) {
			if(!FileSelector->SelectFiles(OpenDialog1, AnsiString(),
					fileNames)) {
				Abort();
			}
		}
		else if(ASender == AddDirectoryM1) {
			AnsiString directory;
			if(!FileSelector->SelectDirectory("Select a directory to add.",
					WideString(), directory)) {
				Abort();
			}
			fileNames->Add(directory);
		}
		else {
			RaiseInvalidOperation();
		}
		FilesDrop(fileNames);
	}
	__finally {
		delete fileNames;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SaveAsM1Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	AnsiString fileName = AnsiString().sprintf("Untitled (%d).csv",
		SavedDocCount + 1);
	if(!FileSelector->SelectFiles(SaveDialog1, fileName, NULL)) {
		Abort();
	}
	TStringList* lines = NULL;
	try {
		lines = new TStringList();
		lines->Add(ListColumnsToCommaText(ActiveListView->Columns));
		for(int i = 0; i < ActiveListView->Items->Count; i++) {
			lines->Add(ListItemToCommaText(ActiveListView->Items->Item[i]));
		}
		lines->SaveToFile(fileName);
		SavedDocCount = SavedDocCount + 1;
	}
	__finally {
		delete lines;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ExitM1Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CutM1Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	CopyM1->Click();
	DeleteM1->Click();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CopyM1Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	TStringList* lines = NULL;
	try {
		lines = new TStringList();
		for(int i = 0; i < ActiveListView->Records->Count; i++) {
			if(ActiveListView->Selected[i]) {
				lines->Add(static_cast<TScanRecord*>(
					ActiveListView->Records->Items[i])->FullName);
			}
		}
		Clipboard()->AsText = lines->Text;
	}
	__finally {
		delete lines;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PasteM1Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	TFileNameList* fileNames = NULL;
	try {
		fileNames = new TFileNameList();
		fileNames->Text = Clipboard()->AsText;
		FilesDrop(fileNames);
	}
	__finally {
		delete fileNames;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DeleteM1Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	ActiveListView->Records->DeleteSelected();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AllM1Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	ActiveListView->SelectAll();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::NoneM1Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	ActiveListView->ClearSelection();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ReverseM1Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	ActiveListView->ReverseSelection();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FirstInGroupM1Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	TIntegerDynArray groups = CountGroups(ActiveListView);
	for(int i = 0; i < ActiveListView->Items->Count; i++) {
		int group = static_cast<TScanRecord*>(
			ActiveListView->Records->Items[i])->Group;
		bool firstInGroup = group >= 0 && groups[group] > 0;
		ActiveListView->Selected[i] = firstInGroup;
		if(firstInGroup) {
			groups[group] = 0;
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SingleInGroupM1Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	TIntegerDynArray groups = CountGroups(ActiveListView);
	for(int i = 0; i < ActiveListView->Items->Count; i++) {
		int group = static_cast<TScanRecord*>(
			ActiveListView->Records->Items[i])->Group;
		ActiveListView->Selected[i] = group >= 0 &&
			groups[group] == 1;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PriorityClick(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	for(int i = 0; i < ActiveListView->Items->Count; i++) {
		if(!ActiveListView->Selected[i]) {
			continue;
		}
		TScanRecord* scanRecord =
			static_cast<TScanRecord*>(ActiveListView->Records->Items[i]);
		if(ASender == SetPrimaryM1) {
			scanRecord->Primary = true;
		}
		else if(ASender == SetSecondaryM1) {
			scanRecord->Primary = false;
		}
		else if(ASender == ToggleM1) {
			scanRecord->Primary = !scanRecord->Primary;
		}
		else {
			RaiseInvalidOperation();
		}
		ActiveListView->UpdateItems(i, i);
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FindM1Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	if(ASender == FindM1) {
		if(!TextFindDialog->Execute()) {
			Abort();
		}
	}
	int itemIndex = TListItemFinder().Find(
		ActiveListView,
		ActiveListView->Items->Count,
		ActiveListView->FocusIndex >= 0 ?
			ActiveListView->FocusIndex :
			0,
		ActiveListView->FocusIndex >= 0 ?
			!ActiveListView->Selected[ActiveListView->FocusIndex] :
			true,
		ASender == FindPreviousM1 ?
			!TextFindDialog->Options.Contains(frDown) :
			TextFindDialog->Options.Contains(frDown),
		true,
		TextFindDialog->Options.Contains(frMatchCase) ?
			CaptionContainsStr :
			CaptionContainsText,
		&(TextFindDialog->FindText));
	if(itemIndex >= 0) {
		ActiveListView->ClearSelection();
		ActiveListView->Focused[itemIndex] = true;
		ActiveListView->Selected[itemIndex] = true;
		ActiveListView->MakeVisible(itemIndex, false);
	}
	else {
		Application->MessageBox(
			"There is no match.",
			Application->Title.c_str(),
			MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND);
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FitM1Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	ActiveListView->FitColumns();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ByNameM1Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	for(int i = 0; i < ScanFields->Count; i++) {
		if(ScanFields->Items[i]->SortMenuItem == ASender) {
			ActiveListView->Records->Sort(CompareScanRecord,
				ScanFields->Items[i]);
			break;
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::InvertM1Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	ActiveListView->Records->Invert();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SizeM1Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	for(int i = 0; i < ScanFields->Count; i++) {
		TScanField* scanField = ScanFields->Items[i];
		if(scanField->ViewMenuItem == ASender) {
			scanField->Visible = !scanField->Visible;
			break;
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ButtonsM1Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	TViewOptions viewOptions = ViewOptions;
	if(ASender == ButtonsM1) {
		viewOptions.ToolBarButtons = !viewOptions.ToolBarButtons;
	}
	else if(ASender == CaptionsM1) {
		viewOptions.ToolBarCaptions = !viewOptions.ToolBarCaptions;
	}
	else if(ASender == StatusBarM1) {
		viewOptions.StatusBar = !viewOptions.StatusBar;
	}
	else if(ASender == PositionsM1) {
		viewOptions.RestorePositions = !viewOptions.RestorePositions;
	}
	else if(ASender == ScanOptionsM1) {
		viewOptions.RestoreScanOptions = !viewOptions.RestoreScanOptions;
	}
	else if(ASender == ExecuteMultipleFilesM1) {
		viewOptions.ConfirmExecuteMultiFiles =
			!viewOptions.ConfirmExecuteMultiFiles;
	}
	else if(ASender == DeleteFilesM1) {
		viewOptions.ConfirmDeleteFiles = !viewOptions.ConfirmDeleteFiles;
	}
	else {
		RaiseInvalidOperation();
	}
	ViewOptions = viewOptions;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::StartM1Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	try {
		// Activate the page of scan targets
		PageControl1->ActivePage = GetPageOf(Targets);

		TScanRecordList* scanTargets = NULL;
		try {
			// Get scan targets
			scanTargets = new TScanRecordList(false);
			GetScanTargets(scanTargets);

			// Get scan options
			TScanOptions scanOptions = ScanOptions;
			UpdateComboBoxHistory(ComboBox11, ComboBox11->DropDownCount);
			UpdateComboBoxHistory(ComboBox12, ComboBox12->DropDownCount);

			// Make a scan thread
			FScanThread = new TScanThread(scanTargets, scanOptions);
			ScanThread->OnMessage = ScanThreadMessage;
			ScanThread->OnProgress = ScanThreadProgress;
			ScanThread->OnResults = ScanThreadResults;
			ScanThread->OnError = ScanThreadError;
			ScanThread->OnTerminate = ScanThreadTerminate;
		}
		__finally {
			delete scanTargets;
		}

		// Display the progress dialog
		ProgressDialog->Caption = "Scan";
		ProgressDialog->Message = AnsiString();
		ProgressDialog->Progress = 0;
		ProgressDialog->Status = AnsiString();
		ProgressDialog->Command = "Cancel";
		ProgressDialog->OnActivate = ProgressDialogActivate;
		ProgressDialog->OnCloseQuery = ProgressDialogCloseQuery;
		ProgressDialog->Execute();
	}
	catch(...) {
		delete FScanThread;
		FScanThread = NULL;
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ClearResultsM1Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	for(int i = 0; i < ListViews->Count; i++) {
		int listViewIndex = ListViews->Count - 1 - i;
		if(IsResultListView(ListViews->Items[listViewIndex])) {
			ListViews->Items[listViewIndex]->Records->Clear();
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::OpenM1Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	TFileNameList* fileHistory = NULL;
	try {
		EnsureExecuteMultiFiles(ActiveListView->SelCount);
		fileHistory = new TFileNameList();
		fileHistory->Duplicates = dupIgnore;
		fileHistory->Sorted = true;
		TFileExecutor fileExecutor;
		fileExecutor.Parent = Application->Handle;
		for(int i = 0; i < ActiveListView->Items->Count; i++) {
			if(!ActiveListView->Selected[i]) {
				continue;
			}

			// Get a file
			AnsiString fileName = static_cast<TScanRecord*>(
				ActiveListView->Records->Items[i])->FullName;
			if(ASender == ExploreM1) {
				if(FileExists(fileName)) {
					fileName = ExtractFileDir(fileName);
				}
			}
			if(!fileHistory->TryAdd(fileName)) {
				continue;
			}

			// Execute the file
			if(ASender == OpenM1) {
				fileExecutor.OpenFile(fileName, AnsiString());
			}
			else if(ASender == ExploreM1) {
				fileExecutor.ExploreFile(fileName, AnsiString());
			}
			else if(ASender == PropertiesM1) {
				fileExecutor.ShowFileProperties(fileName, AnsiString());
			}
			else {
				RaiseInvalidOperation();
			}
		}
	}
	__finally {
		delete fileHistory;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::TotalSizeM1Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	int selectionCount = 0;
	DWORDLONG totalSize = 0;
	for(int i = 0; i < ActiveListView->Records->Count; i++) {
		if(ActiveListView->Selected[i]) {
			selectionCount++;
			totalSize += static_cast<TScanRecord*>(
				ActiveListView->Records->Items[i])->Size;
		}
	}
	Application->MessageBox(
		AnsiString().sprintf(
			"The total size of the %s selected items is %s bytes.",
			FormatInteger(selectionCount).c_str(),
			FormatInteger(totalSize).c_str()).c_str(),
		Application->Title.c_str(),
		MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CopyToDirectoryM1Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	TFileNameList* srcFiles = NULL;
	TFileNameList* dstFiles = NULL;
	try {
		// Get destination files
		if(ASender == CopyToDirectoryM1 || ASender == MoveToDirectoryM1) {
			dstFiles = new TFileNameList();
			GetDstForFileOperation(dstFiles);
		}

		// Get source files
		{
			srcFiles = new TFileNameList();
			GetSrcForFileOperation(srcFiles);
		}

		// Perform a file operation
		if(ASender == MoveToRecycleBinM1) {
			EnsureDeleteFiles(ActiveListView->SelCount);
		}
		TFileOperator fileOperator;
		fileOperator.AllowUndo = true;
		bool fileOpeartionSucceeded;
		bool removeSelected = false;
		if(ASender == CopyToDirectoryM1) {
			fileOpeartionSucceeded =
				fileOperator.CopyFiles(srcFiles, dstFiles);
		}
		else if(ASender == MoveToDirectoryM1) {
			fileOpeartionSucceeded =
				fileOperator.MoveFiles(srcFiles, dstFiles);
			removeSelected = true;
		}
		else if(ASender == MoveToRecycleBinM1) {
			fileOpeartionSucceeded =
				fileOperator.DeleteFiles(srcFiles);
			removeSelected = true;
		}
		else {
			RaiseInvalidOperation();
		}
		if(removeSelected) {
			for(;;) {
				if(!fileOpeartionSucceeded) {
					if(Application->MessageBox(
							"Remove selected items from the list?",
							Application->Title.c_str(),
							MB_YESNO | MB_ICONEXCLAMATION |
								MB_SETFOREGROUND) != IDYES) {
						break;
					}
				}
				ActiveListView->Records->DeleteSelected();
				break;
			}
		}
	}
	__finally {
		delete dstFiles;
		delete srcFiles;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CreateShortcutsM1Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	// Get the destination
	AnsiString directory;
	if(!FileSelector->SelectDirectory(
			"Select the location to create shortcuts.", WideString(),
			directory)) {
		Abort();
	}

	// Create shortcut files
	for(int i = 0; i < ActiveListView->Items->Count; i++) {
		if(!ActiveListView->Selected[i]) {
			continue;
		}
		TScanRecord* scanRecord =
			static_cast<TScanRecord*>(ActiveListView->Records->Items[i]);
		AnsiString fileName = scanRecord->BaseName;
		if(scanRecord->Group >= 0) {
			fileName = AnsiString().sprintf("[%d] ", scanRecord->Group + 1) +
				fileName;
		}
		fileName = IncludeTrailingPathDelimiter(directory) + fileName;
		for(int tryCount = 0;; tryCount++) {
			AnsiString newFileName = fileName;
			if(tryCount > 0) {
				newFileName += AnsiString().sprintf(" (%d)", tryCount + 1);
			}
			newFileName += ".lnk";
			if(!FileOrDirExists(newFileName)) {
				fileName = newFileName;
				break;
			}
		}
		TShortCutFile shortCutFile(fileName);
		shortCutFile.Target = scanRecord->FullName;
		shortCutFile.UpdateFile();
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::HelpM2Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	if(Application->MessageBox(
			AnsiString().sprintf("Open the file \"%s\"?",
			FILE_HELP.c_str()).c_str(),
			Application->Title.c_str(),
			MB_OKCANCEL | MB_ICONEXCLAMATION | MB_SETFOREGROUND) != IDOK) {
		Abort();
	}
	TFileExecutor fileExecutor;
	fileExecutor.Parent = Application->Handle;
	fileExecutor.OpenFile(FILE_HELP, AnsiString());
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AboutM1Click(TObject* ASender)
{
	TAutoCursor busyNotification(crHourGlass);

	TMessageDialog messageDialog;
	messageDialog.Caption = "About";
	messageDialog.Message = INFO_ABOUT;
	messageDialog.Buttons->CommaText = "OK";
	messageDialog.Default = 0;
	messageDialog.Cancel = 0;
	messageDialog.Type = TMessageDialog::TYPE_ORIGINAL;
	messageDialog.Icon->Assign(Application->Icon);
	messageDialog.Execute();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PopupMenuItemClick(TObject* ASender)
{
	ClickMenuItem(ASender);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ToolButtonClick(TObject* ASender)
{
	ClickMenuItem(ASender);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ControlEnter(TObject* ASender)
{
	Change();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ControlExit(TObject* ASender)
{
	Change();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CheckBoxClick(TObject* ASender)
{
	Change();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PageControlChange(TObject* ASender)
{
	Change();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PageControlChanging(TObject* ASender,
	bool& AAllowChange)
{
	if(ASender == GetPageOf(Targets)) {
		EnsureEntriesForScanOptions();
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ListViewColumnClick(TObject* ASender,
	TListColumn* AColumn)
{
	TVirtualListView* listView = dynamic_cast<TVirtualListView*>(ASender);
	static_cast<TScanField*>(
		listView->Fields->Items[AColumn->Index])->SortMenuItem->Click();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ListViewData(TObject* ASender, TListItem* AItem)
{
	TVirtualListView* listView = dynamic_cast<TVirtualListView*>(ASender);
	TScanRecord* scanRecord =
		static_cast<TScanRecord*>(listView->Records->Items[AItem->Index]);
	if(scanRecord != NULL) {
		for(int i = 0; i < listView->Fields->Count; i++) {
			TScanField* scanField =
				static_cast<TScanField*>(listView->Fields->Items[i]);
			if(i == 0) {
				AItem->Caption = scanField->DescribeRecord(scanRecord);
			}
			else {
				AItem->SubItems->Add(scanField->DescribeRecord(scanRecord));
			}
		}
		AItem->ImageIndex = scanRecord->Primary ? 0 : 1;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ListViewDataFind(TObject* ASender,
	TItemFind AFind, const AnsiString AFindString,
	const TPoint& AFindPosition, Pointer AFindData, int AStartIndex,
	TSearchDirection ADirection, bool AWrap, int& AIndex)
{
	if(AFind == ifPartialString || AFind == ifExactString) {
		TAutoCursor busyNotification(crHourGlass);

		AIndex = TListItemFinder().Find(
			ASender,
			dynamic_cast<TVirtualListView*>(ASender)->Items->Count,
			AStartIndex - 1,
			true,
			true,
			AWrap,
			AFind == ifPartialString ?
				CaptionContainsText :
				CaptionStartsWithText,
			&AnsiString(AFindString));
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ListViewDblClick(TObject* ASender)
{
	OpenM1->Click();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ListViewFieldsChange(TObject* ASender)
{
	Change();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ListRecordsChange(TObject* ASender)
{
	Change();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ListRecordsDelete(TObject* ASender, void* AItem)
{
	delete static_cast<TScanRecord*>(AItem);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ListViewChanged(TObject* ASender)
{
	Change();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ClipboardViewerChange(TObject* ASender)
{
	Change();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::TextDialogClose(TObject* ASender,
	TCloseAction& AAction)
{
	Change();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ScanThreadMessage(TObject* ASender,
	AnsiString AMessage)
{
	ProgressDialog->Message = AMessage;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ScanThreadProgress(TObject* ASender, int ADone,
	int AAll)
{
	ProgressDialog->ShowProgress(ADone, AAll);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ScanThreadResults(TObject* ASender,
	TScanRecordList* ADuplicate, TScanRecordList* AUnique,
	TScanRecordList* AError)
{
	SetScanRecordsToListView(Duplicate, ADuplicate);
	SetScanRecordsToListView(Unique, AUnique);
	SetScanRecordsToListView(Error, AError);
	ActivateFirstResultPage();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ScanThreadError(TObject* ASender,
	AnsiString AMessage)
{
	Application->MessageBox(
		AMessage.c_str(),
		Application->Title.c_str(),
		MB_OK | MB_ICONERROR | MB_SETFOREGROUND);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ScanThreadTerminate(TObject* ASender)
{
	FScanThread = NULL;
	ProgressDialog->Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ProgressDialogActivate(TObject* ASender)
{
	ScanThread->Resume();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ProgressDialogCloseQuery(TObject* ASender,
	bool& ACanClose)
{
	if(ScanThread != NULL) {
		ScanThread->Terminate();
		ACanClose = false;
	}
}
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
const AnsiString TMainForm::INFO_ABOUT =
	"Dupy Scan for Windows Version 1.0.0.\r\n\r\n"
	"Copyright (C) 2007 Moochos. All rights reserved.";
const AnsiString TMainForm::FILE_PATH =
	FullyQualifyFileName(ExtractFilePath(Application->ExeName));
const AnsiString TMainForm::FILE_SETTINGS = FILE_PATH + "Settings.ini";
const AnsiString TMainForm::FILE_HELP = FILE_PATH + "Help\\index.html";
//---------------------------------------------------------------------------
__fastcall TMainForm::EInvalidInput::EInvalidInput(AnsiString AMessage) :
	Exception(AMessage)
{
	// Do nothing
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::CaptionContainsStr(void* AList, int AIndex,
	void* AData)
{
	return AnsiContainsStr(
		static_cast<TListView*>(AList)->Items->Item[AIndex]->Caption,
		*static_cast<AnsiString*>(AData));
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::CaptionContainsText(void* AList, int AIndex,
	void* AData)
{
	TVirtualListView* listView = static_cast<TVirtualListView*>(AList);
	AnsiString caption =
		static_cast<TScanField*>(listView->Fields->Items[0])->DescribeRecord(
			static_cast<TScanRecord*>(listView->Records->Items[AIndex]));
	return AnsiContainsText(caption, *static_cast<AnsiString*>(AData));
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::CaptionStartsWithText(void* AList, int AIndex,
	void* AData)
{
	TVirtualListView* listView = static_cast<TVirtualListView*>(AList);
	AnsiString caption =
		static_cast<TScanField*>(listView->Fields->Items[0])->DescribeRecord(
			static_cast<TScanRecord*>(listView->Records->Items[AIndex]));
	return AnsiStartsText(*static_cast<AnsiString*>(AData), caption);
}
//---------------------------------------------------------------------------
int __fastcall TMainForm::CompareScanRecord(void* AItem1, void* AItem2,
	void* AData)
{
	return static_cast<TScanField*>(AData)->CompareRecord(
		static_cast<TScanRecord*>(AItem1),
		static_cast<TScanRecord*>(AItem2));
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Initialize()
{
	try {
		Application->OnIdle = ApplicationIdle;
	 	DragAcceptFiles(Handle, true);
		InitializeMenus();
		InitializeDialogs();
		InitializeListViews();
		InitializeScanFields();
		InitializeFileSelector();
		InitializeClipboardViewer();
		InitializeView();
		InitializeScanOptions();
		FInitialized = true;
		Change();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::InitializeMenus()
{
	FMenuLinks = new TTypedList<TMenuLink>();

	TMenuLink* menuLink;
	MenuLinks->Add(menuLink = new TMenuLink(NewM1));
	menuLink->PopupMenuItem = NULL;
	menuLink->ToolButton = NewT1;

	MenuLinks->Add(menuLink = new TMenuLink(AddFilesM1));
	menuLink->PopupMenuItem = NULL;
	menuLink->ToolButton = AddFilesT1;

	MenuLinks->Add(menuLink = new TMenuLink(SaveAsM1));
	menuLink->PopupMenuItem = NULL;
	menuLink->ToolButton = SaveAsT1;

	MenuLinks->Add(menuLink = new TMenuLink(CutM1));
	menuLink->PopupMenuItem = CutP1;
	menuLink->ToolButton = CutT1;

	MenuLinks->Add(menuLink = new TMenuLink(CopyM1));
	menuLink->PopupMenuItem = CopyP1;
	menuLink->ToolButton = CopyT1;

	MenuLinks->Add(menuLink = new TMenuLink(PasteM1));
	menuLink->PopupMenuItem = PasteP1;
	menuLink->ToolButton = PasteT1;

	MenuLinks->Add(menuLink = new TMenuLink(DeleteM1));
	menuLink->PopupMenuItem = DeleteP1;
	menuLink->ToolButton = DeleteT1;

	MenuLinks->Add(menuLink = new TMenuLink(SetPrimaryM1));
	menuLink->PopupMenuItem = SetPrimaryP1;
	menuLink->ToolButton = SetPrimaryT1;

	MenuLinks->Add(menuLink = new TMenuLink(SetSecondaryM1));
	menuLink->PopupMenuItem = SetSecondaryP1;
	menuLink->ToolButton = NULL;

	MenuLinks->Add(menuLink = new TMenuLink(ToggleM1));
	menuLink->PopupMenuItem = ToggleP1;
	menuLink->ToolButton = NULL;

	MenuLinks->Add(menuLink = new TMenuLink(StartM1));
	menuLink->PopupMenuItem = NULL;
	menuLink->ToolButton = StartT1;

	MenuLinks->Add(menuLink = new TMenuLink(OpenM1));
	menuLink->PopupMenuItem = OpenP1;
	menuLink->ToolButton = NULL;

	MenuLinks->Add(menuLink = new TMenuLink(ExploreM1));
	menuLink->PopupMenuItem = ExploreP1;
	menuLink->ToolButton = NULL;

	MenuLinks->Add(menuLink = new TMenuLink(PropertiesM1));
	menuLink->PopupMenuItem = PropertiesP1;
	menuLink->ToolButton = NULL;

	MenuLinks->Add(menuLink = new TMenuLink(TotalSizeM1));
	menuLink->PopupMenuItem = TotalSizeP1;
	menuLink->ToolButton = NULL;

	MenuLinks->Add(menuLink = new TMenuLink(CopyToDirectoryM1));
	menuLink->PopupMenuItem = CopyToDirectoryP1;
	menuLink->ToolButton = NULL;

	MenuLinks->Add(menuLink = new TMenuLink(MoveToDirectoryM1));
	menuLink->PopupMenuItem = MoveToDirectoryP1;
	menuLink->ToolButton = NULL;

	MenuLinks->Add(menuLink = new TMenuLink(MoveToRecycleBinM1));
	menuLink->PopupMenuItem = MoveToRecycleBinP1;
	menuLink->ToolButton = MoveToRecycleBinT1;

	MenuLinks->Add(menuLink = new TMenuLink(CreateShortcutsM1));
	menuLink->PopupMenuItem = CreateShortcutsP1;
	menuLink->ToolButton = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::InitializeDialogs()
{
	TextFindDialog = new TTextFindDialog(this);
	TextFindDialog->Options =
		TFindOptions() << frDown << frHideWholeWord << frDisableWholeWord;
	TextFindDialog->OnClose = TextDialogClose;

	ProgressDialog = new TProgressDialog(this);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::InitializeListViews()
{
	{
		FListViews = new TTypedList<TVirtualListView>(false);

		ListViews->Add(FTargets = new TVirtualListView(this));
		Targets->Parent = Panel11;

		ListViews->Add(FDuplicate = new TVirtualListView(this));
		Duplicate->Parent = TabSheet2;

		ListViews->Add(FUnique = new TVirtualListView(this));
		Unique->Parent = TabSheet3;

		ListViews->Add(FError = new TVirtualListView(this));
		Error->Parent = TabSheet4;
	}

	{
		FResultListViews = new TTypedList<TVirtualListView>(false);
		ResultListViews->Add(Duplicate);
		ResultListViews->Add(Unique);
		ResultListViews->Add(Error);
	}

	for(int i = 0; i < ListViews->Count; i++) {
		TVirtualListView* listView = ListViews->Items[i];
		listView->Align = alClient;
		listView->MultiSelect = true;
		listView->PopupMenu = PopupMenu1;
		listView->ReadOnly = true;
		listView->RowSelect = true;
		listView->SmallImages = ImageList3;
		listView->ViewStyle = vsReport;
		listView->OnColumnClick = ListViewColumnClick;
		listView->OnData = ListViewData;
		listView->OnDataFind = ListViewDataFind;
		listView->OnDblClick = ListViewDblClick;
		listView->OnEnter = ControlEnter;
		listView->OnExit = ControlExit;
		listView->Fields->OnChange = ListViewFieldsChange;
		listView->Records->OnChange = ListRecordsChange;
		listView->Records->OnDelete = ListRecordsDelete;
		listView->OnChanged = ListViewChanged;
		if(IsResultListView(listView)) {
			GetPageOf(listView)->TabVisible = false;
		}
	}

	ActiveListView = Targets;
	ActiveControl = Targets;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::InitializeScanFields()
{
	FScanFields = new TTypedList<TScanField>();

	TScanField* scanField;
	ScanFields->Add(scanField = new TScanBaseNameField());
	scanField->SortMenuItem = ByNameM1;
	scanField->ViewMenuItem = NULL;
	scanField->ListViews->Add(Targets);
	scanField->ListViews->Add(Duplicate);
	scanField->ListViews->Add(Unique);
	scanField->ListViews->Add(Error);

	ScanFields->Add(scanField = new TScanDirectoryField());
	scanField->SortMenuItem = ByDirectoryM1;
	scanField->ViewMenuItem = NULL;
	scanField->ListViews->Add(Targets);
	scanField->ListViews->Add(Duplicate);
	scanField->ListViews->Add(Unique);
	scanField->ListViews->Add(Error);

	ScanFields->Add(scanField = new TScanSizeField());
	scanField->SortMenuItem = BySizeM1;
	scanField->ViewMenuItem = SizeM1;
	scanField->ListViews->Add(Duplicate);
	scanField->ListViews->Add(Unique);

	ScanFields->Add(scanField = new TScanTypeField());
	scanField->SortMenuItem = ByTypeM1;
	scanField->ViewMenuItem = TypeM1;
	scanField->ListViews->Add(Duplicate);
	scanField->ListViews->Add(Unique);

	ScanFields->Add(scanField = new TScanModificationTimeField());
	scanField->SortMenuItem = ByLastModifiedM1;
	scanField->ViewMenuItem = LastModifiedM1;
	scanField->ListViews->Add(Duplicate);
	scanField->ListViews->Add(Unique);

	ScanFields->Add(scanField = new TScanHashField());
	scanField->SortMenuItem = ByHashM1;
	scanField->ViewMenuItem = HashM1;
	scanField->ListViews->Add(Duplicate);

	ScanFields->Add(scanField = new TScanPrimaryField());
	scanField->SortMenuItem = ByPriorityM1;
	scanField->ViewMenuItem = NULL;
	scanField->ListViews->Add(Targets);
	scanField->ListViews->Add(Duplicate);
	scanField->ListViews->Add(Unique);
	scanField->ListViews->Add(Error);

	ScanFields->Add(scanField = new TScanGroupField());
	scanField->SortMenuItem = ByGroupM1;
	scanField->ViewMenuItem = NULL;
	scanField->ListViews->Add(Duplicate);

	ScanFields->Add(scanField = new TScanErrorField());
	scanField->SortMenuItem = ByErrorM1;
	scanField->ViewMenuItem = NULL;
	scanField->ListViews->Add(Error);

	for(int i = 0; i < ScanFields->Count; i++) {
		if(ScanFields->Items[i]->ViewMenuItem == NULL) {
			ScanFields->Items[i]->Visible = true;
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::InitializeFileSelector()
{
	FFileSelector = new TFileSelector();
	FOpenDialogs = new TTypedList<TOpenDialog>(false);
	OpenDialogs->Add(OpenDialog1);
	OpenDialogs->Add(SaveDialog1);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::InitializeClipboardViewer()
{
	FClipboardViewer = new TClipboardViewer();
	ClipboardViewer->OnChange = ClipboardViewerChange;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::InitializeView()
{
	Panel11->Align = alClient;
	Panel12->Align = alBottom;
	PageControl1->Align = alClient;
	Panel1->Align = alClient;

	TViewOptions defViewOptions;
	defViewOptions.ToolBarButtons = true;
	defViewOptions.ToolBarCaptions = false;
	defViewOptions.StatusBar = true;
	defViewOptions.RestorePositions = false;
	defViewOptions.RestoreScanOptions = false;
	defViewOptions.ConfirmExecuteMultiFiles = true;
	defViewOptions.ConfirmDeleteFiles = true;
	ViewOptions = defViewOptions;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::InitializeScanOptions()
{
	TScanOptions defScanOptions;
	defScanOptions.InclusiveFileExtensions = AnsiString();
	defScanOptions.ExclusiveFileExtensions = AnsiString();
	defScanOptions.ExclusiveFileAttributes = 0;
	defScanOptions.Properties = TScanOptions::PROPERTY_TYPE;
	defScanOptions.HashAlgorithm = TScanOptions::HASH_ALGORITHM_SHA_1;
	ScanOptions = defScanOptions;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Uninitialize()
{
	UninitializeClipboardViewer();
	UninitializeFileSelector();
	UninitializeScanFields();
	UninitializeListViews();
	UninitializeMenus();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::UninitializeMenus()
{
	delete FMenuLinks;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::UninitializeListViews()
{
	delete FResultListViews;
	delete FListViews;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::UninitializeScanFields()
{
	delete FScanFields;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::UninitializeFileSelector()
{
	delete FOpenDialogs;
	delete FFileSelector;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::UninitializeClipboardViewer()
{
	delete FClipboardViewer;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ProcessCommandParams()
{
	for(;;) {
		if(ParamCount() <= 0) {
			break;
		}
		TFileNameList* fileNames = new TFileNameList();
		try {
			fileNames->Capacity = ParamCount();
			for(int i = 0; i < ParamCount(); i++) {
				fileNames->Add(ParamStr(i + 1));
			}
			FilesDrop(fileNames);
		}
		__finally {
			delete fileNames;
		}
		break;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ClickMenuItem(TObject* ASender)
{
	for(int i = 0; i < MenuLinks->Count; i++) {
		if(MenuLinks->Items[i]->IsControlAssociated(ASender)) {
			MenuLinks->Items[i]->MenuItem->Click();
			break;
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FilesDrop(TStrings* AFileNames)
{
	class EInvalidFile : public Exception
	{
	public:
		__fastcall EInvalidFile(AnsiString AMessage) : Exception(AMessage) {}
	};

	TScanRecordList* scanRecords = NULL;
	TFileNameList* fileHistory = NULL;
	TFileNameList* fileNames = NULL;
	try {
		// Get filenames
		fileHistory = new TFileNameList();
		fileHistory->Duplicates = dupIgnore;
		fileHistory->Sorted = true;
		for(int i = 0; i < ActiveListView->Records->Count; i++) {
			fileHistory->Add(static_cast<TScanRecord*>(
				ActiveListView->Records->Items[i])->FullName);
		}
		fileNames = new TFileNameList();
		fileNames->Duplicates = dupIgnore;
		fileNames->Sorted = true;
		for(int i = 0; i < AFileNames->Count; i++) {
			try {
				AnsiString fileName = FullyQualifyFileName(
					AFileNames->Strings[i]);
				if(fileName.Length() <= 0) {
					throw EInvalidFile(AnsiString().sprintf(
						"The filename \"%s\" is invalid.\r\n\r\nContinue?",
						AFileNames->Strings[i].c_str()));
				}
				if(!fileHistory->TryAdd(fileName)) {
					throw EInvalidFile(AnsiString().sprintf(
						"The file \"%s\" is already in the list.\r\n\r\n"
						"Continue?", fileName.c_str()));
				}
				fileNames->Add(fileName);
			}
			catch(EInvalidFile& e) {
				if(Application->MessageBox(
						e.Message.c_str(),
						Application->Title.c_str(),
						MB_YESNO | MB_ICONERROR | MB_SETFOREGROUND) !=
							IDYES) {
					Abort();
				}
			}
		}
		fileHistory->Clear();

		// Make scan records
		scanRecords = new TScanRecordList();
		for(int i = 0; i < fileNames->Count; i++) {
			scanRecords->Add(new TScanRecord(fileNames->Strings[i]));
		}
		fileNames->Clear();

		// Add the scan records to the Targets list view
		AddScanRecordsToListView(Targets, scanRecords);
		ActiveListView = Targets;
		ActiveControl = Targets;
	}
	__finally {
		delete fileHistory;
		delete fileNames;
		delete scanRecords;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EnsureExecuteMultiFiles(int AFileCount)
{
	if(AFileCount > 1) {
		if(Application->MessageBox(
				AnsiString().sprintf("Execute %d files?", AFileCount).c_str(),
				Application->Title.c_str(),
				MB_YESNO | MB_ICONEXCLAMATION | MB_SETFOREGROUND) != IDYES) {
			Abort();
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EnsureDeleteFiles(int AFileCount)
{
	if(AFileCount > 0) {
		if(Application->MessageBox(
				AnsiString().sprintf("Move %d files to the recycle bin?",
					AFileCount).c_str(),
				Application->Title.c_str(),
				MB_YESNO | MB_ICONEXCLAMATION | MB_SETFOREGROUND) != IDYES) {
			Abort();
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::GetScanTargets(TScanRecordList* AScanTargets)
{
	AScanTargets->Clear();
	AScanTargets->Capacity = Targets->Records->Count;
	bool primaryIncluded = false;
	for(int i = 0; i < Targets->Records->Count; i++) {
		TScanRecord* scanRecord =
			static_cast<TScanRecord*>(Targets->Records->Items[i]);
		if(!primaryIncluded && scanRecord->Primary) {
			primaryIncluded = true;
		}
		AScanTargets->Add(scanRecord);
	}
	if(!primaryIncluded) {
		RaiseInvalidInput("Specify primary items", Targets);
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::GetDstForFileOperation(TStrings* AFiles)
{
	AnsiString directory;
	if(!FileSelector->SelectDirectory("Select the destination.",
			WideString(), directory)) {
		Abort();
	}
	AFiles->Clear();
	AFiles->Add(directory);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::GetSrcForFileOperation(TStrings* AFiles)
{
	TFileNameList* files = NULL;
	try {
		files = new TFileNameList();
		files->Duplicates = dupIgnore;
		files->Sorted = true;
		for(int i = 0; i < ActiveListView->Items->Count; i++) {
			if(!ActiveListView->Selected[i]) {
				continue;
			}
			AnsiString fileName = static_cast<TScanRecord*>(
				ActiveListView->Records->Items[i])->FullName;
			if(!FileOrDirExists(fileName)) {
				throw Exception(AnsiString().sprintf(
					"The file or directory \"%s\" does not exist",
					fileName.c_str()));
			}
			if(!files->TryAdd(fileName)) {
				throw Exception(AnsiString().sprintf(
					"The filename \"%s\" duplicates",
					fileName.c_str()));
			}
		}
		AFiles->Clear();
		for(int i = 0; i < files->Count; i++) {
			AFiles->Add(files->Strings[files->Count - 1 - i]);
		}
	}
	__finally {
		delete files;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetActiveListView(TVirtualListView* AListView)
{
	PageControl1->ActivePage = GetPageOf(AListView);
}
//---------------------------------------------------------------------------
TMainForm::TViewOptions __fastcall TMainForm::GetViewOptions()
{
	TViewOptions viewOptions;
	viewOptions.ToolBarButtons = ToolBar1->Visible;
	viewOptions.ToolBarCaptions = ToolBar1->ShowCaptions;
	viewOptions.StatusBar = StatusBar1->Visible;
	viewOptions.RestorePositions = PositionsM1->Checked;
	viewOptions.RestoreScanOptions = ScanOptionsM1->Checked;
	viewOptions.ConfirmExecuteMultiFiles = ExecuteMultipleFilesM1->Checked;
	viewOptions.ConfirmDeleteFiles = DeleteFilesM1->Checked;
	return viewOptions;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetViewOptions(TViewOptions AViewOptions)
{
	ToolBar1->Visible = AViewOptions.ToolBarButtons;
	ToolBar1->ShowCaptions = AViewOptions.ToolBarCaptions;
	if(!AViewOptions.ToolBarCaptions) {
		ToolBar1->ButtonWidth = 23;
		ToolBar1->ButtonHeight = 22;
	}
	StatusBar1->Visible = AViewOptions.StatusBar;

	ButtonsM1->Checked = AViewOptions.ToolBarButtons;
	CaptionsM1->Checked = AViewOptions.ToolBarCaptions;
	StatusBarM1->Checked = AViewOptions.StatusBar;
	PositionsM1->Checked = AViewOptions.RestorePositions;
	ScanOptionsM1->Checked = AViewOptions.RestoreScanOptions;
	ExecuteMultipleFilesM1->Checked = AViewOptions.ConfirmExecuteMultiFiles;
	DeleteFilesM1->Checked = AViewOptions.ConfirmDeleteFiles;
}
//---------------------------------------------------------------------------
TScanOptions __fastcall TMainForm::GetScanOptions()
{
	TScanOptions scanOptions;
	scanOptions.InclusiveFileExtensions = ComboBox11->Text;
	scanOptions.ExclusiveFileExtensions = ComboBox12->Text;
	scanOptions.ExclusiveFileAttributes = 0;
	if(CheckBox11->Checked) {
		scanOptions.ExclusiveFileAttributes |= FILE_ATTRIBUTE_READONLY;
	}
	if(CheckBox12->Checked) {
		scanOptions.ExclusiveFileAttributes |= FILE_ATTRIBUTE_HIDDEN;
	}
	if(CheckBox13->Checked) {
		scanOptions.ExclusiveFileAttributes |= FILE_ATTRIBUTE_ARCHIVE;
	}
	if(CheckBox14->Checked) {
		scanOptions.ExclusiveFileAttributes |= FILE_ATTRIBUTE_SYSTEM;
	}
	scanOptions.Properties = 0;
	if(CheckBox15->Checked) {
		scanOptions.Properties |= TScanOptions::PROPERTY_BASE_NAME;
	}
	if(CheckBox16->Checked) {
		scanOptions.Properties |= TScanOptions::PROPERTY_TYPE;
	}
	if(CheckBox17->Checked) {
		scanOptions.Properties |= TScanOptions::PROPERTY_MODIFICATION_TIME;
	}
	switch(ComboBox13->ItemIndex) {
	case 0:
		scanOptions.HashAlgorithm = TScanOptions::HASH_ALGORITHM_MD5;
		break;
	case 1:
		scanOptions.HashAlgorithm = TScanOptions::HASH_ALGORITHM_SHA_1;
		break;
	case 2:
		scanOptions.HashAlgorithm = TScanOptions::HASH_ALGORITHM_SHA_256;
		break;
	default:
		RaiseInvalidInput("Select a hash algorithm", ComboBox13);
	}
	return scanOptions;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetScanOptions(TScanOptions AScanOptions)
{
	ComboBox11->Text = AScanOptions.InclusiveFileExtensions;
	ComboBox12->Text = AScanOptions.ExclusiveFileExtensions;
	CheckBox11->Checked = (AScanOptions.ExclusiveFileAttributes &
		FILE_ATTRIBUTE_READONLY) != 0;
	CheckBox12->Checked = (AScanOptions.ExclusiveFileAttributes &
		FILE_ATTRIBUTE_HIDDEN) != 0;
	CheckBox13->Checked = (AScanOptions.ExclusiveFileAttributes &
		FILE_ATTRIBUTE_ARCHIVE) != 0;
	CheckBox14->Checked = (AScanOptions.ExclusiveFileAttributes &
		FILE_ATTRIBUTE_SYSTEM) != 0;
	CheckBox15->Checked = (AScanOptions.Properties &
		TScanOptions::PROPERTY_BASE_NAME) != 0;
	CheckBox16->Checked = (AScanOptions.Properties &
		TScanOptions::PROPERTY_TYPE) != 0;
	CheckBox17->Checked = (AScanOptions.Properties &
		TScanOptions::PROPERTY_MODIFICATION_TIME) != 0;
	switch(AScanOptions.HashAlgorithm) {
	case TScanOptions::HASH_ALGORITHM_MD5:
		ComboBox13->ItemIndex = 0;
		break;
	case TScanOptions::HASH_ALGORITHM_SHA_1:
		ComboBox13->ItemIndex = 1;
		break;
	case TScanOptions::HASH_ALGORITHM_SHA_256:
		ComboBox13->ItemIndex = 2;
		break;
	default:
		ComboBox13->ItemIndex = -1;
		break;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EnsureEntriesForScanOptions()
{
	GetScanOptions();
}
//---------------------------------------------------------------------------
AnsiString __fastcall TMainForm::GetStatus()
{
	return StatusBar1->SimpleText;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetStatus(AnsiString AStatus)
{
	if(AStatus != StatusBar1->SimpleText) {
		StatusBar1->SimpleText = AStatus;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Change()
{
	if(Initialized) {
		UpdateActiveListView();
		UpdateMenuItemStates();
		UpdatePageStates();
		UpdateStatus();
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::UpdateActiveListView()
{
	TVirtualListView* newActiveListView = NULL;
	for(int i = 0; i < ListViews->Count; i++) {
		if(GetPageOf(ListViews->Items[i]) == PageControl1->ActivePage) {
			newActiveListView = ListViews->Items[i];
			break;
		}
	}
	FActiveListView = newActiveListView;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::UpdateMenuItemStates()
{
	{
		bool recordsActive = ActiveControl == ActiveListView;
		bool recordsAvailable = ActiveListView->Items->Count > 0;
		bool recordsSelected = ActiveListView->SelCount > 0;
		bool findTextAvailable = TextFindDialog->FindText.Length() > 0;
		bool resultsAvailable = false;
		for(int i = 0; i < ListViews->Count; i++) {
			if(IsResultListView(ListViews->Items[i]) &&
					ListViews->Items[i]->Records->Count > 0) {
				resultsAvailable = true;
				break;
			}
		}

		SaveAsM1->Enabled =
			recordsAvailable;

		CutM1->Enabled =
			recordsActive && recordsSelected;
		CopyM1->Enabled =
			recordsActive && recordsSelected;
		PasteM1->Enabled =
			recordsActive && ActiveListView == Targets &&
			Clipboard()->HasFormat(CF_TEXT);
		DeleteM1->Enabled =
			recordsActive && recordsSelected;

		AllM1->Enabled =
			recordsActive && recordsAvailable;
		NoneM1->Enabled =
			recordsActive && recordsAvailable;
		ReverseM1->Enabled =
			recordsActive && recordsAvailable;
		FirstInGroupM1->Enabled =
			recordsActive && recordsAvailable &&
			ActiveListView == Duplicate;
		SingleInGroupM1->Enabled =
			recordsActive && recordsAvailable &&
			ActiveListView == Duplicate;

		SetPrimaryM1->Enabled =
			recordsActive && recordsSelected && ActiveListView == Targets;
		SetSecondaryM1->Enabled =
			recordsActive && recordsSelected && ActiveListView == Targets;
		ToggleM1->Enabled =
			recordsActive && recordsSelected && ActiveListView == Targets;

		FindM1->Enabled =
			recordsActive && recordsAvailable;
		FindNextM1->Enabled =
			recordsActive && recordsAvailable && findTextAvailable;
		FindPreviousM1->Enabled =
			recordsActive && recordsAvailable && findTextAvailable;

		FitM1->Enabled =
			ActiveListView->Columns->Count > 0;
		for(int i = 0; i < ScanFields->Count; i++) {
			TScanField* scanField = ScanFields->Items[i];
			if(scanField->SortMenuItem != NULL) {
				bool scanFieldAvailable =
					ActiveListView->Fields->Find(scanField) >= 0;
				scanField->SortMenuItem->Enabled = scanFieldAvailable;
				scanField->SortMenuItem->Visible = scanFieldAvailable;
			}
			if(scanField->ViewMenuItem != NULL) {
				scanField->ViewMenuItem->Checked = scanField->Visible;
			}
		}

		StartM1->Enabled =
			Targets->Items->Count > 0;
		ClearResultsM1->Enabled =
			resultsAvailable;

		OpenM1->Enabled =
			recordsActive && recordsSelected;
		ExploreM1->Enabled =
			recordsActive && recordsSelected;
		PropertiesM1->Enabled =
			recordsActive && recordsSelected;
		TotalSizeM1->Enabled =
			recordsActive && recordsSelected &&
			(ActiveListView == Duplicate || ActiveListView == Unique);
		CopyToDirectoryM1->Enabled =
			recordsActive && recordsSelected;
		MoveToDirectoryM1->Enabled =
			recordsActive && recordsSelected;
		MoveToRecycleBinM1->Enabled =
			recordsActive && recordsSelected;
		CreateShortcutsM1->Enabled =
			recordsActive && recordsSelected;
	}

	for(int i = 0; i < MenuLinks->Count; i++) {
		MenuLinks->Items[i]->UpdateControlStates();
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::UpdatePageStates()
{
	for(int i = 0; i < ListViews->Count; i++) {
		if(IsResultListView(ListViews->Items[i])) {
			GetPageOf(ListViews->Items[i])->TabVisible =
				ListViews->Items[i]->Items->Count > 0;
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::UpdateStatus()
{
	Status = ActiveListView->SelCount <= 0 ?
		AnsiString().sprintf(
			"%s items",
			FormatInteger(ActiveListView->Records->Count).c_str()) :
		AnsiString().sprintf(
			"%s items are selected",
			FormatInteger(ActiveListView->SelCount).c_str());
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMDropFiles(TMessage& AMessage)
{
	TFileNameList* fileNames = NULL;
	try {
		HDROP dropHandle = reinterpret_cast<HDROP>(AMessage.WParam);
		try {
			TAutoCursor busyNotification(crHourGlass);
			SetForegroundWindow(Application->Handle);

			UINT fileCount = DragQueryFile(dropHandle, 0xFFFFFFFF, NULL, 0);
			fileNames = new TFileNameList();
			for(UINT i = 0; i < fileCount; i++) {
				TCHAR* buffer = NULL;
				try {
					UINT bufferSize = DragQueryFile(dropHandle, i, NULL, 0) +
						1;
					buffer = new TCHAR[bufferSize];
					ZeroMemory(buffer, bufferSize);
					DragQueryFile(dropHandle, i, buffer, bufferSize);
					fileNames->Add(buffer);
				}
				__finally {
					delete[] buffer;
				}
			}
		}
		__finally {
			DragFinish(dropHandle);
		}
		FilesDrop(fileNames);
	}
	__finally {
		delete fileNames;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::RaiseInvalidInput(AnsiString AMessage,
	TWinControl* AControl) const
{
	if(AControl != NULL && AControl->CanFocus()) {
		AControl->SetFocus();
	}
	throw EInvalidInput(AMessage);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::RaiseInvalidOperation() const
{
	throw EInvalidOperation("Invalid opeartion");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetScanRecordsToListView(
	TVirtualListView* AListView, TScanRecordList* AScanRecords) const
{
	AListView->Records->Clear();
	AListView->Records->AddItems(AScanRecords);
	AScanRecords->ExtractAll();
	if(AListView->Records->Count > 0) {
		AListView->Focused[0] = true;
		AListView->MakeVisible(0, false);
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::AddScanRecordsToListView(
	TVirtualListView* AListView, TScanRecordList* AScanRecords) const
{
	int oldCount = AListView->Records->Count;
	AListView->ClearSelection();
	AListView->Records->AddItems(AScanRecords);
	AScanRecords->ExtractAll();
	int newCount = AListView->Records->Count;
	if(newCount > 0) {
		AListView->Focused[oldCount] = true;
		AListView->SelectItems(oldCount, newCount - oldCount, true);
		AListView->MakeVisible(oldCount, false);
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ActivateFirstResultPage()
{
	for(int i = 0; i < ListViews->Count; i++) {
		if(IsResultListView(ListViews->Items[i]) &&
				GetPageOf(ListViews->Items[i])->TabVisible) {
			ActiveListView = ListViews->Items[i];
			ActiveControl = ListViews->Items[i];
			break;
		}
	}
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::IsResultListView(
	TVirtualListView* AListView) const
{
	return ResultListViews->IndexOf(AListView) >= 0;
}
//---------------------------------------------------------------------------
TTabSheet* __fastcall TMainForm::GetPageOf(TControl* AControl) const
{
	TTabSheet* page = NULL;
	TControl* curControl = AControl;
	for(;;) {
		if(curControl == NULL) {
			break;
		}
		if(curControl->InheritsFrom(__classid(TTabSheet))) {
			page = dynamic_cast<TTabSheet*>(curControl);
			break;
		}
		curControl = curControl->Parent;
	}
	return page;
}
//---------------------------------------------------------------------------
TIntegerDynArray __fastcall TMainForm::CountGroups(
	TVirtualListView* AListView) const
{
	TIntegerDynArray groups;
	int maxGroup = -1;
	for(int i = 0; i < AListView->Records->Count; i++) {
		TScanRecord* scanRecord = static_cast<TScanRecord*>(
			AListView->Records->Items[i]);
		if(scanRecord->Group > maxGroup) {
			maxGroup = scanRecord->Group;
		}
	}
	groups.Length = maxGroup + 1;
	for(int i = 0; i < groups.Length; i++) {
		groups[i] = 0;
	}
	for(int i = 0; i < AListView->Records->Count; i++) {
		TScanRecord* scanRecord = static_cast<TScanRecord*>(
			AListView->Records->Items[i]);
		if(scanRecord->Group >= 0) {
			groups[scanRecord->Group]++;
		}
	}
	return groups;
}
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* AOwner) :
	TForm(AOwner),
	FMenuLinks(NULL),
	FListViews(NULL),
	FResultListViews(NULL),
	FTargets(NULL),
	FDuplicate(NULL),
	FUnique(NULL),
	FError(NULL),
	FActiveListView(NULL),
	FScanFields(NULL),
	FFileSelector(NULL),
	FOpenDialogs(NULL),
	FClipboardViewer(NULL),
	FScanThread(NULL),
	FInitialized(false),
	FSavedDocCount(0)
{
	Initialize();
	try {
		LoadSettings();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
bool __fastcall TMainForm::IsShortCut(TWMKey& AMessage)
{
	bool isShortCut = false;
	TMenu* menus[] = {Menu, PopupMenu};
	TShortCut shortCut;
	{
		TShiftState shiftState;
		if(GetAsyncKeyState(VK_SHIFT) < 0) {
			shiftState << ssShift;
		}
		if(GetAsyncKeyState(VK_CONTROL) < 0) {
			shiftState << ssCtrl;
		}
		if(GetAsyncKeyState(VK_MENU) < 0) {
			shiftState << ssAlt;
		}
		shortCut = ShortCut(AMessage.CharCode, shiftState);
	}
	for(int i = 0; i < sizeof(menus) / sizeof(TMenu*); i++) {
		if(menus[i] == NULL) {
			continue;
		}
		TMenuItem* menuItem = menus[i]->FindItem(shortCut, fkShortCut);
		if(menuItem == NULL || !menuItem->Enabled) {
			continue;
		}
		menuItem->Click();
		isShortCut = true;
		break;
	}
	return isShortCut;
}
//---------------------------------------------------------------------------

