//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "TMainForm.h"
#include "TQuoteIniFile.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
// Internal
//---------------------------------------------------------------------------
namespace
{
const char* INI_SCANFIELDS = "ScanFields";
const char* INI_SCANFIELDS_VISIBLE = "Visible";
const char* INI_OPENDIALOGS = "OpenDialogs";
const char* INI_OPENDIALOGS_INITIALDIR = "InitialDir";
const char* INI_OPENDIALOGS_FILTERINDEX = "FilterIndex";
const char* INI_VIEWOPTIONS = "ViewOptions";
const char* INI_VIEWOPTIONS_TOOLBARBUTTONS = "ToolBarButtons";
const char* INI_VIEWOPTIONS_TOOLBARCAPTIONS = "ToolBarCaptions";
const char* INI_VIEWOPTIONS_STATUSBAR = "StatusBar";
const char* INI_VIEWOPTIONS_RESTOREPOSITIONS = "RestorePositions";
const char* INI_VIEWOPTIONS_RESTORESCANOPTIONS = "RestoreScanOptions";
const char* INI_VIEWOPTIONS_CONFIRMEXECUTEMULTIFILES =
	"ConfirmExecuteMultiFiles";
const char* INI_VIEWOPTIONS_CONFIRDELETEFILES = "ConfirmDeleteFiles";
const char* INI_POSITIONS = "Positions";
const char* INI_POSITIONS_SCREENWIDTH = "ScreenWidth";
const char* INI_POSITIONS_SCREENHEIGHT = "ScreenHeight";
const char* INI_POSITIONS_WINDOWSTATE = "WindowState";
const char* INI_POSITIONS_LEFT = "Left";
const char* INI_POSITIONS_TOP = "Top";
const char* INI_POSITIONS_RIGHT = "Right";
const char* INI_POSITIONS_BOTTOM = "Bottom";
const char* INI_SCANOPTIONS = "ScanOptions";
const char* INI_SCANOPTIONS_INCLUSIVEFILEEXTENSIONS =
	"InclusiveFileExtensions";
const char* INI_SCANOPTIONS_EXCLUSIVEFILEEXTENSIONS =
	"ExclusiveFileExtensions";
const char* INI_SCANOPTIONS_EXCLUSIVEFILEATTRIBUTES =
	"ExclusiveFileAttributes";
const char* INI_SCANOPTIONS_PROPERTIES = "Properties";
const char* INI_SCANOPTIONS_HASHALGORITHM = "HashAlgorithm";
const char* INI_HISTORIES = "Histories";
const char* INI_HISTORIES_INCLUSIVEFILEEXTENSIONHISTORY =
	"InclusiveFileExtensionHistory";
const char* INI_HISTORIES_EXCLUSIVEFILEEXTENSIONHISTORY =
	"ExclusiveFileExtensionHistory";
} // End of namespace
//---------------------------------------------------------------------------
// External
//---------------------------------------------------------------------------
void __fastcall TMainForm::LoadSettings()
{
	TQuoteIniFile* iniFile = NULL;
	try {
		iniFile = new TQuoteIniFile(FILE_SETTINGS);
		for(int i = 0; i < ScanFields->Count; i++) {
			ScanFields->Items[i]->Visible = iniFile->ReadBool(
				INI_SCANFIELDS,
				INI_SCANFIELDS_VISIBLE + AnsiString(i),
				ScanFields->Items[i]->Visible);
		}
		{
			FileSelector->InitialDir = iniFile->ReadQuotedString(
				INI_OPENDIALOGS,
				INI_OPENDIALOGS_INITIALDIR,
				FileSelector->InitialDir);
			for(int i = 0; i < OpenDialogs->Count; i++) {
				OpenDialogs->Items[i]->FilterIndex = iniFile->ReadInteger(
					INI_OPENDIALOGS,
					INI_OPENDIALOGS_FILTERINDEX + AnsiString(i),
					OpenDialogs->Items[i]->FilterIndex);
			}
		}
		{
			TViewOptions viewOptions = ViewOptions;
			viewOptions.ToolBarButtons = iniFile->ReadBool(
				INI_VIEWOPTIONS,
				INI_VIEWOPTIONS_TOOLBARBUTTONS,
				viewOptions.ToolBarButtons);
			viewOptions.ToolBarCaptions = iniFile->ReadBool(
				INI_VIEWOPTIONS,
				INI_VIEWOPTIONS_TOOLBARCAPTIONS,
				viewOptions.ToolBarCaptions);
			viewOptions.StatusBar = iniFile->ReadBool(
				INI_VIEWOPTIONS,
				INI_VIEWOPTIONS_STATUSBAR,
				viewOptions.StatusBar);
			viewOptions.RestorePositions = iniFile->ReadBool(
				INI_VIEWOPTIONS,
				INI_VIEWOPTIONS_RESTOREPOSITIONS,
				viewOptions.RestorePositions);
			viewOptions.RestoreScanOptions = iniFile->ReadBool(
				INI_VIEWOPTIONS,
				INI_VIEWOPTIONS_RESTORESCANOPTIONS,
				viewOptions.RestoreScanOptions);
			viewOptions.ConfirmExecuteMultiFiles = iniFile->ReadBool(
				INI_VIEWOPTIONS,
				INI_VIEWOPTIONS_CONFIRMEXECUTEMULTIFILES,
				viewOptions.ConfirmExecuteMultiFiles);
			viewOptions.ConfirmDeleteFiles = iniFile->ReadBool(
				INI_VIEWOPTIONS,
				INI_VIEWOPTIONS_CONFIRDELETEFILES,
				viewOptions.ConfirmDeleteFiles);
			ViewOptions = viewOptions;
		}
		if(ViewOptions.RestorePositions) {
			for(;;) {
				if(iniFile->ReadInteger(
						INI_POSITIONS,
						INI_POSITIONS_SCREENWIDTH,
						Screen->Width) != Screen->Width) {
					break;
				}
				if(iniFile->ReadInteger(
						INI_POSITIONS,
						INI_POSITIONS_SCREENHEIGHT,
						Screen->Height) != Screen->Height) {
					break;
				}
				if(iniFile->ReadInteger(
						INI_POSITIONS,
						INI_POSITIONS_WINDOWSTATE,
						WindowState) != wsNormal) {
					break;
				}
				TRect newBoundsRect = BoundsRect;
				newBoundsRect.left = iniFile->ReadInteger(
					INI_POSITIONS,
					INI_POSITIONS_LEFT,
					newBoundsRect.left);
				newBoundsRect.top = iniFile->ReadInteger(
					INI_POSITIONS,
					INI_POSITIONS_TOP,
					newBoundsRect.top);
				newBoundsRect.right = iniFile->ReadInteger(
					INI_POSITIONS,
					INI_POSITIONS_RIGHT,
					newBoundsRect.right);
				newBoundsRect.bottom = iniFile->ReadInteger(
					INI_POSITIONS,
					INI_POSITIONS_BOTTOM,
					newBoundsRect.bottom);
				BoundsRect = newBoundsRect;
				for(int i = 0; i < ListViews->Count; i++) {
					ListViews->Items[i]->FitColumns();
				}
				break;
			}
		}
		if(ViewOptions.RestoreScanOptions) {
			TScanOptions scanOptions = ScanOptions;
			scanOptions.InclusiveFileExtensions = iniFile->ReadQuotedString(
				INI_SCANOPTIONS,
				INI_SCANOPTIONS_INCLUSIVEFILEEXTENSIONS,
				scanOptions.InclusiveFileExtensions);
			scanOptions.ExclusiveFileExtensions = iniFile->ReadQuotedString(
				INI_SCANOPTIONS,
				INI_SCANOPTIONS_EXCLUSIVEFILEEXTENSIONS,
				scanOptions.ExclusiveFileExtensions);
			scanOptions.ExclusiveFileAttributes = iniFile->ReadInteger(
				INI_SCANOPTIONS,
				INI_SCANOPTIONS_EXCLUSIVEFILEATTRIBUTES,
				scanOptions.ExclusiveFileAttributes);
			scanOptions.Properties = iniFile->ReadInteger(
				INI_SCANOPTIONS,
				INI_SCANOPTIONS_PROPERTIES,
				scanOptions.Properties);
			scanOptions.HashAlgorithm = iniFile->ReadInteger(
				INI_SCANOPTIONS,
				INI_SCANOPTIONS_HASHALGORITHM,
				scanOptions.HashAlgorithm);
			ScanOptions = scanOptions;
		}
		{
			ComboBox11->Items->CommaText = iniFile->ReadQuotedString(
				INI_HISTORIES,
				INI_HISTORIES_INCLUSIVEFILEEXTENSIONHISTORY,
				ComboBox11->Items->CommaText);
			ComboBox12->Items->CommaText = iniFile->ReadQuotedString(
				INI_HISTORIES,
				INI_HISTORIES_EXCLUSIVEFILEEXTENSIONHISTORY,
				ComboBox12->Items->CommaText);
		}
	}
	__finally {
		delete iniFile;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SaveSettings()
{
	TQuoteIniFile* iniFile = NULL;
	try {
		iniFile = new TQuoteIniFile(FILE_SETTINGS);
		for(int i = 0; i < ScanFields->Count; i++) {
			iniFile->WriteBool(
				INI_SCANFIELDS,
				INI_SCANFIELDS_VISIBLE + AnsiString(i),
				ScanFields->Items[i]->Visible);
		}
		{
			iniFile->WriteQuotedString(
				INI_OPENDIALOGS,
				INI_OPENDIALOGS_INITIALDIR,
				FileSelector->InitialDir);
			for(int i = 0; i < OpenDialogs->Count; i++) {
				iniFile->WriteInteger(
					INI_OPENDIALOGS,
					INI_OPENDIALOGS_FILTERINDEX + AnsiString(i),
					OpenDialogs->Items[i]->FilterIndex);
			}
		}
		{
			TViewOptions viewOptions = ViewOptions;
			iniFile->WriteBool(
				INI_VIEWOPTIONS,
				INI_VIEWOPTIONS_TOOLBARBUTTONS,
				viewOptions.ToolBarButtons);
			iniFile->WriteBool(
				INI_VIEWOPTIONS,
				INI_VIEWOPTIONS_TOOLBARCAPTIONS,
				viewOptions.ToolBarCaptions);
			iniFile->WriteBool(
				INI_VIEWOPTIONS,
				INI_VIEWOPTIONS_STATUSBAR,
				viewOptions.StatusBar);
			iniFile->WriteBool(
				INI_VIEWOPTIONS,
				INI_VIEWOPTIONS_RESTOREPOSITIONS,
				viewOptions.RestorePositions);
			iniFile->WriteBool(
				INI_VIEWOPTIONS,
				INI_VIEWOPTIONS_RESTORESCANOPTIONS,
				viewOptions.RestoreScanOptions);

			iniFile->WriteBool(
				INI_VIEWOPTIONS,
				INI_VIEWOPTIONS_CONFIRMEXECUTEMULTIFILES,
				viewOptions.ConfirmExecuteMultiFiles);
			iniFile->WriteBool(
				INI_VIEWOPTIONS,
				INI_VIEWOPTIONS_CONFIRDELETEFILES,
				viewOptions.ConfirmDeleteFiles);
		}
		{
			iniFile->WriteInteger(
				INI_POSITIONS,
				INI_POSITIONS_SCREENWIDTH,
				Screen->Width);
			iniFile->WriteInteger(
				INI_POSITIONS,
				INI_POSITIONS_SCREENHEIGHT,
				Screen->Height);
			iniFile->WriteInteger(
				INI_POSITIONS,
				INI_POSITIONS_WINDOWSTATE,
				static_cast<TWindowState>(WindowState));
			iniFile->WriteInteger(
				INI_POSITIONS,
				INI_POSITIONS_LEFT,
				BoundsRect.left);
			iniFile->WriteInteger(
				INI_POSITIONS,
				INI_POSITIONS_TOP,
				BoundsRect.top);
			iniFile->WriteInteger(
				INI_POSITIONS,
				INI_POSITIONS_RIGHT,
				BoundsRect.right);
			iniFile->WriteInteger(
				INI_POSITIONS,
				INI_POSITIONS_BOTTOM,
				BoundsRect.bottom);
		}
		{
			TScanOptions scanOptions = ScanOptions;
			iniFile->WriteQuotedString(
				INI_SCANOPTIONS,
				INI_SCANOPTIONS_INCLUSIVEFILEEXTENSIONS,
				scanOptions.InclusiveFileExtensions);
			iniFile->WriteQuotedString(
				INI_SCANOPTIONS,
				INI_SCANOPTIONS_EXCLUSIVEFILEEXTENSIONS,
				scanOptions.ExclusiveFileExtensions);
			iniFile->WriteInteger(
				INI_SCANOPTIONS,
				INI_SCANOPTIONS_EXCLUSIVEFILEATTRIBUTES,
				scanOptions.ExclusiveFileAttributes);
			iniFile->WriteInteger(
				INI_SCANOPTIONS,
				INI_SCANOPTIONS_PROPERTIES,
				scanOptions.Properties);
			iniFile->WriteInteger(
				INI_SCANOPTIONS,
				INI_SCANOPTIONS_HASHALGORITHM,
				scanOptions.HashAlgorithm);
		}
		{
			iniFile->WriteQuotedString(
				INI_HISTORIES,
				INI_HISTORIES_INCLUSIVEFILEEXTENSIONHISTORY,
				ComboBox11->Items->CommaText);
			iniFile->WriteQuotedString(
				INI_HISTORIES,
				INI_HISTORIES_EXCLUSIVEFILEEXTENSIONHISTORY,
				ComboBox12->Items->CommaText);
		}
		iniFile->UpdateFile();
	}
	__finally {
		delete iniFile;
	}
}
//---------------------------------------------------------------------------

