//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "TScanComparer.h"
#include "TScanRecord.h"
#include "SystemUtils.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
// class TScanComparer : public TObject
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TScanComparer::TScanComparer() :
	TObject()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TScanComparer::~TScanComparer()
{
	// Do nothing
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// class TScanBaseNameComparer : public TScanComparer
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
bool __fastcall TScanBaseNameComparer::EnsureRecord(
	TScanRecord* ARecord) const
{
	return !ARecord->HasError();
}
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TScanBaseNameComparer::TScanBaseNameComparer() :
	TScanComparer()
{
	// Do nothing
}
//---------------------------------------------------------------------------
int __fastcall TScanBaseNameComparer::CompareRecord(TScanRecord* ARecord1,
	TScanRecord* ARecord2, int ADefault)
{
	int difference = ADefault;
	if(EnsureRecord(ARecord1) && EnsureRecord(ARecord2)) {
		difference = AnsiCompareFileName(
			ARecord1->BaseName,
			ARecord2->BaseName);
	}
	return difference;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// class TScanSizeComparer : public TScanComparer
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
bool __fastcall TScanSizeComparer::EnsureRecord(
	TScanRecord* ARecord) const
{
	return !ARecord->HasError();
}
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TScanSizeComparer::TScanSizeComparer() :
	TScanComparer()
{
	// Do nothing
}
//---------------------------------------------------------------------------
int __fastcall TScanSizeComparer::CompareRecord(TScanRecord* ARecord1,
	TScanRecord* ARecord2, int ADefault)
{
	int difference = ADefault;
	if(EnsureRecord(ARecord1) && EnsureRecord(ARecord2)) {
		difference = ARecord1->Size < ARecord2->Size ?
			-1 :
			(ARecord1->Size > ARecord2->Size ? 1 : 0);
	}
	return difference;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// class TScanTypeComparer : public TScanComparer
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
bool __fastcall TScanTypeComparer::EnsureRecord(TScanRecord* ARecord) const
{
	if(!ARecord->HasError()) {
		if(ARecord->Type.Length() == 0) {
			ARecord->Error = TScanRecord::ERROR_UNKNOWN_FILE_TYPE;
		}
	}
	return !ARecord->HasError();
}
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TScanTypeComparer::TScanTypeComparer() :
	TScanComparer()
{
	// Do nothing
}
//---------------------------------------------------------------------------
int __fastcall TScanTypeComparer::CompareRecord(TScanRecord* ARecord1,
	TScanRecord* ARecord2, int ADefault)
{
	int difference = ADefault;
	if(EnsureRecord(ARecord1) && EnsureRecord(ARecord2)) {
		difference = AnsiCompareStr(ARecord1->Type, ARecord2->Type);
	}
	return difference;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// class TScanModificationTimeComparer : public TScanComparer
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
bool __fastcall TScanModificationTimeComparer::EnsureRecord(
	TScanRecord* ARecord) const
{
	if(!ARecord->HasError()) {
		if(!IsValidFileTime(ARecord->ModificationTime)) {
			ARecord->Error = TScanRecord::ERROR_UNKNOWN_FILE_TIME;
		}
	}
	return !ARecord->HasError();
}
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TScanModificationTimeComparer::TScanModificationTimeComparer() :
	TScanComparer()
{
	// Do nothing
}
//---------------------------------------------------------------------------
int __fastcall TScanModificationTimeComparer::CompareRecord(
	TScanRecord* ARecord1, TScanRecord* ARecord2, int ADefault)
{
	int difference = ADefault;
	if(EnsureRecord(ARecord1) && EnsureRecord(ARecord2)) {
		difference = CompareFileTime(
			&(ARecord1->ModificationTime),
			&(ARecord2->ModificationTime));
	}
	return difference;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// class TScanHashComparer : public TScanComparer
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
bool __fastcall TScanHashComparer::EnsureRecord(TScanRecord* ARecord)
{
	if(!ARecord->HasError()) {
		if(ARecord->Hash.Length() == 0) {
			AnsiString hash;
			if(DoHash(ARecord->FullName, hash)) {
				ARecord->Hash = hash;
			}
			else {
				ARecord->Error = TScanRecord::ERROR_CANNOT_GET_HASH;
			}
		}
	}
	return !ARecord->HasError();
}
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
bool __fastcall TScanHashComparer::DoHash(AnsiString AFileName,
	AnsiString& AHash)
{
	bool succeeded = false;
	if(OnHash != NULL) {
		OnHash(this, AFileName, AHash, succeeded);
	}
	return succeeded;
}
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TScanHashComparer::TScanHashComparer() :
	TScanComparer(),
	FOnHash(NULL)
{
	// Do nothing
}
//---------------------------------------------------------------------------
int __fastcall TScanHashComparer::CompareRecord(TScanRecord* ARecord1,
	TScanRecord* ARecord2, int ADefault)
{
	int difference = ADefault;
	if(EnsureRecord(ARecord1) && EnsureRecord(ARecord2)) {
		difference = AnsiCompareText(ARecord1->Hash, ARecord2->Hash);
	}
	return difference;
}
//---------------------------------------------------------------------------

