//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "TScanField.h"
#include "TScanRecord.h"
#include "SystemUtils.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
// class TScanField
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
int __fastcall TScanField::CompareScanFieldByPriority(void* AItem1,
	void* AItem2, void* AData)
{
	return
		static_cast<TScanField*>(AItem1)->Priority -
		static_cast<TScanField*>(AItem2)->Priority;
}
//---------------------------------------------------------------------------
void __fastcall TScanField::Initialize()
{
	try {
		FListViews = new TTypedList<TVirtualListView>(false);
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanField::Uninitialize()
{
	delete FListViews;
}
//---------------------------------------------------------------------------
bool __fastcall TScanField::GetVisible()
{
	bool visible = false;
	for(int i = 0; i < ListViews->Count; i++) {
		if(ListViews->Items[i]->Fields->Find(this) >= 0) {
			visible = true;
			break;
		}
	}
	return visible;
}
//---------------------------------------------------------------------------
void __fastcall TScanField::SetVisible(bool AVisible)
{
	for(int i = 0; i < ListViews->Count; i++) {
		TVirtualListView* listView = ListViews->Items[i];
		if(AVisible) {
			if(listView->Fields->Find(this) >= 0) {
				continue;
			}
			int itemIndex = listView->Fields->Count;
			for(int j = 0; j < listView->Fields->Count; j++) {
				if(static_cast<TScanField*>(
						listView->Fields->Items[j])->Priority > Priority) {
					itemIndex = j;
					break;
				}
			}
			listView->Fields->Insert(itemIndex, static_cast<void*>(this));
			listView->Columns->Items[itemIndex]->Alignment = Alignment;
			listView->Columns->Items[itemIndex]->Caption = Caption;
		}
		else {
			if(listView->Fields->Find(this) < 0) {
				continue;
			}
			listView->Fields->Remove(this);
		}
		listView->FitColumns();
	}
}
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TScanField::TScanField() :
	FPriority(0),
	FCaption(),
	FAlignment(taLeftJustify),
	FSortMenuItem(NULL),
	FViewMenuItem(NULL),
	FListViews(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TScanField::~TScanField()
{
	Uninitialize();
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// class TScanBaseNameField : public TScanField
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TScanBaseNameField::TScanBaseNameField() :
	TScanField()
{
	Priority = 0;
	Caption = "Name";
	Alignment = taLeftJustify;
}
//---------------------------------------------------------------------------
__fastcall TScanBaseNameField::~TScanBaseNameField()
{
	// Do nothing
}
//---------------------------------------------------------------------------
AnsiString __fastcall TScanBaseNameField::DescribeRecord(
	TScanRecord* ARecord)
{
	return ARecord->BaseName;
}
//---------------------------------------------------------------------------
int __fastcall TScanBaseNameField::CompareRecord(TScanRecord* ARecord1,
	TScanRecord* ARecord2)
{
	return AnsiCompareStr(ARecord1->BaseName, ARecord2->BaseName);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// class TScanDirectoryField : public TScanField
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TScanDirectoryField::TScanDirectoryField() :
	TScanField()
{
	Priority = 1;
	Caption = "Directory";
	Alignment = taLeftJustify;
}
//---------------------------------------------------------------------------
__fastcall TScanDirectoryField::~TScanDirectoryField()
{
	// Do nothing
}
//---------------------------------------------------------------------------
AnsiString __fastcall TScanDirectoryField::DescribeRecord(
	TScanRecord* ARecord)
{
	return ARecord->Directory;
}
//---------------------------------------------------------------------------
int __fastcall TScanDirectoryField::CompareRecord(TScanRecord* ARecord1,
	TScanRecord* ARecord2)
{
	return AnsiCompareStr(ARecord1->Directory, ARecord2->Directory);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// class TScanSizeField : public TScanField
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TScanSizeField::TScanSizeField() :
	TScanField()
{
	Priority = 2;
	Caption = "Size";
	Alignment = taRightJustify;
}
//---------------------------------------------------------------------------
__fastcall TScanSizeField::~TScanSizeField()
{
	// Do nothing
}
//---------------------------------------------------------------------------
AnsiString __fastcall TScanSizeField::DescribeRecord(
	TScanRecord* ARecord)
{
	return FormatInteger(ARecord->Size);
}
//---------------------------------------------------------------------------
int __fastcall TScanSizeField::CompareRecord(TScanRecord* ARecord1,
	TScanRecord* ARecord2)
{
	return ARecord1->Size < ARecord2->Size ?
		-1 :
		(ARecord1->Size > ARecord2->Size ? 1 : 0);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// class TScanTypeField : public TScanField
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TScanTypeField::TScanTypeField() :
	TScanField()
{
	Priority = 3;
	Caption = "Type";
	Alignment = taLeftJustify;
}
//---------------------------------------------------------------------------
__fastcall TScanTypeField::~TScanTypeField()
{
	// Do nothing
}
//---------------------------------------------------------------------------
AnsiString __fastcall TScanTypeField::DescribeRecord(
	TScanRecord* ARecord)
{
	return ARecord->Type;
}
//---------------------------------------------------------------------------
int __fastcall TScanTypeField::CompareRecord(TScanRecord* ARecord1,
	TScanRecord* ARecord2)
{
	return AnsiCompareStr(ARecord1->Type, ARecord2->Type);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// class TScanModificationTimeField : public TScanField
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TScanModificationTimeField::TScanModificationTimeField() :
	TScanField()
{
	Priority = 4;
	Caption = "Last Modified";
	Alignment = taLeftJustify;
}
//---------------------------------------------------------------------------
__fastcall TScanModificationTimeField::~TScanModificationTimeField()
{
	// Do nothing
}
//---------------------------------------------------------------------------
AnsiString __fastcall TScanModificationTimeField::DescribeRecord(
	TScanRecord* ARecord)
{
	AnsiString description = "(Unknown)";
	for(;;) {
		if(!IsValidFileTime(ARecord->ModificationTime)) {
			break;
		}
		FILETIME localFileTime;
		if(!FileTimeToLocalFileTime(&(ARecord->ModificationTime),
				&localFileTime)) {
			break;
		}
		description = FormatFileTime(localFileTime);
		break;
	}
	return description;
}
//---------------------------------------------------------------------------
int __fastcall TScanModificationTimeField::CompareRecord(
	TScanRecord* ARecord1, TScanRecord* ARecord2)
{
	return CompareFileTime(
		&(ARecord1->ModificationTime),
		&(ARecord2->ModificationTime));
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// class TScanHashField : public TScanField
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TScanHashField::TScanHashField() :
	TScanField()
{
	Priority = 5;
	Caption = "Hash";
	Alignment = taLeftJustify;
}
//---------------------------------------------------------------------------
__fastcall TScanHashField::~TScanHashField()
{
	// Do nothing
}
//---------------------------------------------------------------------------
AnsiString __fastcall TScanHashField::DescribeRecord(
	TScanRecord* ARecord)
{
	return ARecord->Hash;
}
//---------------------------------------------------------------------------
int __fastcall TScanHashField::CompareRecord(TScanRecord* ARecord1,
	TScanRecord* ARecord2)
{
	return AnsiCompareStr(ARecord1->Hash, ARecord2->Hash);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// class TScanPrimaryField : public TScanField
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TScanPrimaryField::TScanPrimaryField() :
	TScanField()
{
	Priority = 6;
	Caption = "Priority";
	Alignment = taLeftJustify;
}
//---------------------------------------------------------------------------
__fastcall TScanPrimaryField::~TScanPrimaryField()
{
	// Do nothing
}
//---------------------------------------------------------------------------
AnsiString __fastcall TScanPrimaryField::DescribeRecord(
	TScanRecord* ARecord)
{
	return ARecord->Primary ? "Primary" : "Secondary";
}
//---------------------------------------------------------------------------
int __fastcall TScanPrimaryField::CompareRecord(TScanRecord* ARecord1,
	TScanRecord* ARecord2)
{
	return (ARecord1->Primary ? 0 : 1) - (ARecord2->Primary ? 0 : 1);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// class TScanGroupField : public TScanField
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TScanGroupField::TScanGroupField() :
	TScanField()
{
	Priority = 7;
	Caption = "Group";
	Alignment = taLeftJustify;
}
//---------------------------------------------------------------------------
__fastcall TScanGroupField::~TScanGroupField()
{
	// Do nothing
}
//---------------------------------------------------------------------------
AnsiString __fastcall TScanGroupField::DescribeRecord(
	TScanRecord* ARecord)
{
	return ARecord->Group + 1;
}
//---------------------------------------------------------------------------
int __fastcall TScanGroupField::CompareRecord(TScanRecord* ARecord1,
	TScanRecord* ARecord2)
{
	return ARecord1->Group - ARecord2->Group;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// class TScanErrorField : public TScanField
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TScanErrorField::TScanErrorField() :
	TScanField()
{
	Priority = 8;
	Caption = "Error";
	Alignment = taLeftJustify;
}
//---------------------------------------------------------------------------
__fastcall TScanErrorField::~TScanErrorField()
{
	// Do nothing
}
//---------------------------------------------------------------------------
AnsiString __fastcall TScanErrorField::DescribeRecord(TScanRecord* ARecord)
{
	AnsiString description;
	switch(ARecord->Error) {
	case TScanRecord::ERROR_NONE:
		description = "None";
		break;
	case TScanRecord::ERROR_CANNOT_FIND:
		description = "Cannot find the file";
		break;
	case TScanRecord::ERROR_ALREADY_ADDED:
		description = "Already added";
		break;
	case TScanRecord::ERROR_CANNOT_GET_FILE_TYPE:
		description = "Cannot get the file type";
		break;
	case TScanRecord::ERROR_UNKNOWN_FILE_TYPE:
		description = "Unknown file type";
		break;
	case TScanRecord::ERROR_UNKNOWN_FILE_TIME:
		description = "Unknown file time";
		break;
	case TScanRecord::ERROR_CANNOT_GET_HASH:
		description = "Cannot get the hash";
		break;
	default:
		description = "(Unknown)";
		break;
	}
	return description;
}
//---------------------------------------------------------------------------
int __fastcall TScanErrorField::CompareRecord(TScanRecord* ARecord1,
	TScanRecord* ARecord2)
{
	return ARecord1->Error - ARecord2->Error;
}
//---------------------------------------------------------------------------

