//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "TScanFileFilter.h"
#include "TFileNameList.h"
#include "TStringTokenizer.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
// class TScanFileFilter : public TObject
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
const char TScanFileFilter::FILE_EXTENSION_DELIMITER = ';';
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
void __fastcall TScanFileFilter::Initialize(TScanOptions AOptions)
{
	try {
		FFileFilters = new TTypedList<TFileFilter>();
		{
			TFileExtensionFilter* inclusiveFileFilter = NULL;
			TFileExtensionFilter* exclusiveFileFilter = NULL;
			try {
				inclusiveFileFilter = MakeFileExtensionFilter(
					AOptions.InclusiveFileExtensions, true);
				exclusiveFileFilter = MakeFileExtensionFilter(
					AOptions.ExclusiveFileExtensions, false);
				if(inclusiveFileFilter != NULL &&
						exclusiveFileFilter != NULL) {
					inclusiveFileFilter->EnsurePair(exclusiveFileFilter);
				}
				FileFilters->Add(inclusiveFileFilter);
				FileFilters->Add(exclusiveFileFilter);
			}
			catch(...) {
				delete exclusiveFileFilter;
				delete inclusiveFileFilter;
				throw;
			}
		}
		FileFilters->Add(MakeFileAttributeFilter(
			AOptions.ExclusiveFileAttributes, false));
		FileFilters->Pack();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanFileFilter::Uninitialize()
{
	delete FFileFilters;
}
//---------------------------------------------------------------------------
TFileExtensionFilter* __fastcall TScanFileFilter::MakeFileExtensionFilter(
	AnsiString AFileExtensions, bool AInclusive) const
{
	TFileExtensionFilter* fileExtensionFilter = NULL;
	try {
		TFileNameList* fileExtensions = NULL;
		try {
			fileExtensions = new TFileNameList();
  				for(TStringTokenizer tokenizer(AFileExtensions,
					FILE_EXTENSION_DELIMITER); tokenizer.HasNext();) {
				AnsiString fileExtension = tokenizer.GetNext();
				if(fileExtension.Length() == 0) {
					continue;
				}
				fileExtensions->Add(IncludeLeadingChar(fileExtension, '.'));
			}
			if(fileExtensions->Count > 0) {
				fileExtensionFilter =
					new TFileExtensionFilter(fileExtensions, AInclusive);
			}
		}
		__finally {
			delete fileExtensions;
		}
	}
	catch(...) {
		delete fileExtensionFilter;
		throw;
	}
	return fileExtensionFilter;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TScanFileFilter::IncludeLeadingChar(AnsiString AString,
	char AChar) const
{
	AnsiString newString = AString;
	if(*newString.c_str() != AChar) {
		newString.Insert(AChar, 1);
	}
	return newString;
}
//---------------------------------------------------------------------------
TFileAttributeFilter* __fastcall TScanFileFilter::MakeFileAttributeFilter(
	DWORD AFileAttributes, bool AInclusive) const
{
	TFileAttributeFilter* fileAttributeFilter = NULL;
	try {
		if(AFileAttributes != 0) {
			fileAttributeFilter =
				new TFileAttributeFilter(AFileAttributes, AInclusive);
		}
	}
	catch(...) {
		delete fileAttributeFilter;
		throw;
	}
	return fileAttributeFilter;
}
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TScanFileFilter::TScanFileFilter(TScanOptions AOptions) :
	TObject(),
	FFileFilters(NULL)
{
	Initialize(AOptions);
}
//---------------------------------------------------------------------------
__fastcall TScanFileFilter::~TScanFileFilter()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
bool __fastcall TScanFileFilter::Passes(const WIN32_FIND_DATA& AFindData)
{
	bool passes = true;
	for(int i = 0; i < FileFilters->Count; i++) {
		if(!FileFilters->Items[i]->Passes(AFindData)) {
			passes = false;
			break;
		}
	}
	return passes;
}
//---------------------------------------------------------------------------

