//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "TScanTargetLister.h"
#include "TFileEnumerator.h"
#include "TFileNameList.h"
#include "TFileTypeList.h"
#include "TScanFileFilter.h"
#include "SystemUtils.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
// class TScanTargetLister : public TObject
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
void __fastcall TScanTargetLister::Initialize(TScanRecordList* ASrcRecords,
	TScanOptions AOptions)
{
	try {
		FSrcRecords = new TScanRecordList();
		SrcRecords->AddExtractions(ASrcRecords);
		for(int i = 0; i < SrcRecords->Count / 2; i++) {
			SrcRecords->Exchange(i, SrcRecords->Count - 1 - i);
		}
		FDstRecords = new TScanRecordList();
		FFileFilter = new TScanFileFilter(AOptions);
		FFileHistory = new TFileNameList();
		FileHistory->Duplicates = dupIgnore;
		FileHistory->Sorted = true;
		FFileTypes = new TFileTypeList();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanTargetLister::Uninitialize()
{
	delete FFileTypes;
	delete FFileHistory;
	delete FFileFilter;
	delete FDstRecords;
	delete FSrcRecords;
}
//---------------------------------------------------------------------------
bool __fastcall TScanTargetLister::ListNext()
{
	bool succeeded = false;
	for(;;) {
		if(SrcRecords->Count <= 0) {
			break;
		}
		TScanRecord srcRecord = *SrcRecords->Last();
		SrcRecords->Delete(SrcRecords->Count - 1);
		if(!FileHistory->TryAdd(srcRecord.FullName)) {
			DstRecords->Add(new TScanRecord(MakeRecord(
				srcRecord.FullName,
				NULL,
				srcRecord.Primary,
				TScanRecord::ERROR_ALREADY_ADDED)));
			break;
		}
		TFileEnumerator fileEnumerator(srcRecord.FullName);
		if(fileEnumerator.HasNext()) {
			AnsiString findPath = ExtractFilePath(srcRecord.FullName);
			do {
				WIN32_FIND_DATA findData = fileEnumerator.GetNext();
				if(lstrcmp(findData.cFileName, ".") == 0 ||
						lstrcmp(findData.cFileName, "..") == 0) {
					continue;
				}
				AnsiString fileName = findPath + findData.cFileName;
				if(!SameFileName(fileName, srcRecord.FullName)) {
					if(!FileHistory->TryAdd(fileName)) {
						DstRecords->Add(new TScanRecord(MakeRecord(
							fileName,
							NULL,
							srcRecord.Primary,
							TScanRecord::ERROR_ALREADY_ADDED)));
						continue;
					}
				}
				if((findData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) !=
						0) {
					SrcRecords->Add(new TScanRecord(MakeRecord(
						IncludeTrailingPathDelimiter(fileName) + "*",
						NULL,
						srcRecord.Primary,
						TScanRecord::ERROR_NONE)));
					continue;
				}
				if(!FileFilter->Passes(findData)) {
					continue;
				}
				DstRecords->Add(new TScanRecord(MakeRecord(
					fileName,
					&findData,
					srcRecord.Primary,
					TScanRecord::ERROR_NONE)));
			} while(fileEnumerator.HasNext());
		}
		else {
			DstRecords->Add(new TScanRecord(MakeRecord(
				srcRecord.FullName,
				NULL,
				srcRecord.Primary,
				TScanRecord::ERROR_CANNOT_FIND)));
		}
		succeeded = true;
		break;
	}
	return succeeded;
}
//---------------------------------------------------------------------------
TScanRecord __fastcall TScanTargetLister::MakeRecord(AnsiString AFullName,
	const WIN32_FIND_DATA* AFindData, bool APrimary, int AError) const
{
	TScanRecord record;
	record.FullName = AFullName;
	if(AFindData != NULL) {
		record.Size = MakeULargeInteger(
			AFindData->nFileSizeLow,
			AFindData->nFileSizeHigh).QuadPart;
		record.Type = FileTypes->Items[AFullName];
		record.ModificationTime = AFindData->ftLastWriteTime;
	}
	record.Primary = APrimary;
	record.Error = AError;
	return record;
}
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TScanTargetLister::TScanTargetLister(TScanRecordList* ASrcRecords,
		TScanOptions AOptions) :
	TWorker(),
	FSrcRecords(NULL),
	FDstRecords(NULL),
	FFileFilter(NULL),
	FFileHistory(NULL),
	FFileTypes(NULL)
{
	Initialize(ASrcRecords, AOptions);
}
//---------------------------------------------------------------------------
__fastcall TScanTargetLister::~TScanTargetLister()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TScanTargetLister::List(TScanRecordList* ADstRecords)
{
	while(ListNext()) {
		DoContinue();
	}
	ADstRecords->Clear();
	ADstRecords->AddExtractions(DstRecords);
}
//---------------------------------------------------------------------------

