//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "TScanThread.h"
#include "TScanResultLister.h"
#include "TScanTargetLister.h"
#include "SystemUtils.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
// class TScanThread : public TThread
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
void __fastcall TScanThread::Initialize(TScanRecordList* ATargets)
{
	try {
		FreeOnTerminate = true;
		FTargets = new TScanRecordList();
		for(int i = 0; i < ATargets->Count; i++) {
			Targets->Add(new TScanRecord(*ATargets->Items[i]));
		}
		FDuplicate = new TScanRecordList();
		FUnique = new TScanRecordList();
		FError = new TScanRecordList();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanThread::Uninitialize()
{
	delete FError;
	delete FUnique;
	delete FDuplicate;
	delete FTargets;
}
//---------------------------------------------------------------------------
void __fastcall TScanThread::Scan()
{
	ListTargets();
	ListResults();
	DoResults(Duplicate, Unique, Error);
}
//---------------------------------------------------------------------------
void __fastcall TScanThread::ListTargets()
{
	DoMessage("Listing targets...");
	TScanTargetLister* targetLister = NULL;
	try {
		targetLister = new TScanTargetLister(Targets, Options);
		targetLister->OnContinue = WorkerContinue;
		targetLister->List(Targets);
	}
	__finally {
		delete targetLister;
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanThread::ListResults()
{
	DoMessage("Listing results...");
	TScanResultLister* resultLister = NULL;
	try {
		resultLister = new TScanResultLister(Targets, Options);
		resultLister->OnContinue = WorkerContinue;
		resultLister->OnProgress = ScanResultListerProgress;
		resultLister->List(Duplicate, Unique, Error);
	}
	__finally {
		delete resultLister;
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanThread::WorkerContinue(TObject* ASender)
{
	if(Terminated) {
		Abort();
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanThread::ScanResultListerProgress(TObject* ASender,
	int ADone, int AAll)
{
	for(;;) {
		if(AAll == 0) {
			break;
		}
		int curProgress = (static_cast<__int64>(ADone) * 100) / AAll;
		if(curProgress == LastProgress) {
			break;
		}
		DoProgress(ADone, AAll);
		FLastProgress = curProgress;
		break;
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanThread::DoMessage(AnsiString AMessage)
{
	if(OnMessage != NULL) {
		try {
			TMessageEventParams params;
			params.Message = AMessage;
			SyncronizeData = &params;
			Synchronize(DoMessageInMainThread);
		}
		__finally {
			SyncronizeData = NULL;
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanThread::DoMessageInMainThread()
{
	if(OnMessage != NULL) {
		TMessageEventParams& params =
			*static_cast<TMessageEventParams*>(SyncronizeData);
		OnMessage(this, params.Message);
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanThread::DoProgress(int ADone, int AAll)
{
	if(OnProgress != NULL) {
		try {
			TProgressEventParams params;
			params.Done = ADone;
			params.All = AAll;
			SyncronizeData = &params;
			Synchronize(DoProgressInMainThread);
		}
		__finally {
			SyncronizeData = NULL;
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanThread::DoProgressInMainThread()
{
	if(OnProgress != NULL) {
		TProgressEventParams& params =
			*static_cast<TProgressEventParams*>(SyncronizeData);
		OnProgress(this, params.Done, params.All);
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanThread::DoResults(TScanRecordList* ADuplicate,
	TScanRecordList* AUnique, TScanRecordList* AError)
{
	if(OnResults != NULL) {
		try {
			TResultsEventParams params;
			params.Duplicate = ADuplicate;
			params.Unique = AUnique;
			params.Error = AError;
			SyncronizeData = &params;
			Synchronize(DoResultsInMainThread);
		}
		__finally {
			SyncronizeData = NULL;
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanThread::DoResultsInMainThread()
{
	if(OnResults != NULL) {
		TResultsEventParams& params =
			*static_cast<TResultsEventParams*>(SyncronizeData);
		OnResults(this, params.Duplicate, params.Unique, params.Error);
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanThread::DoError(AnsiString AMessage)
{
	if(OnError != NULL) {
		try {
			TMessageEventParams params;
			params.Message = AMessage;
			SyncronizeData = &params;
			Synchronize(DoErrorInMainThread);
		}
		__finally {
			SyncronizeData = NULL;
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TScanThread::DoErrorInMainThread()
{
	if(OnError != NULL) {
		TMessageEventParams& params =
			*static_cast<TMessageEventParams*>(SyncronizeData);
		OnError(this, params.Message);
	}
}
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
void __fastcall TScanThread::Execute()
{
	try {
		Scan();
	}
	catch(EAbort& e) {
		// Do nothing
	}
	catch(Exception& e) {
		DoError(e.Message);
	}
	catch(...) {
		DoError("Unknown error occured");
	}
}
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TScanThread::TScanThread(TScanRecordList* ATargets,
		TScanOptions AOptions) :
	TThread(true),
	FTargets(NULL),
	FDuplicate(NULL),
	FUnique(NULL),
	FError(NULL),
	FOptions(AOptions),
	FLastProgress(0),
	FSyncronizeData(NULL),
	FOnMessage(NULL),
	FOnProgress(NULL),
	FOnResults(NULL),
	FOnError(NULL)
{
	Initialize(ATargets);
}
//---------------------------------------------------------------------------
__fastcall TScanThread::~TScanThread()
{
	Uninitialize();
}
//---------------------------------------------------------------------------

