//---------------------------------------------------------------------------
#ifndef TScanThreadH
#define TScanThreadH
#include <Classes.hpp>
#include "TScanOptions.h"
#include "TScanRecord.h"
//---------------------------------------------------------------------------
class TScanThread : public TThread
{
public:
	typedef void __fastcall (__closure *TMessageEvent)(TObject* ASender,
		AnsiString AMessage);
	typedef void __fastcall (__closure *TProgressEvent)(TObject* ASender,
		int ADone, int AAll);
	typedef void __fastcall (__closure *TResultsEvent)(TObject* ASender,
		TScanRecordList* ADuplicate, TScanRecordList* AUnique,
		TScanRecordList* AError);
private:
	struct TMessageEventParams
	{
		AnsiString Message;
	};
	struct TProgressEventParams
	{
		int Done;
		int All;
	};
	struct TResultsEventParams
	{
		TScanRecordList* Duplicate;
		TScanRecordList* Unique;
		TScanRecordList* Error;
	};
	TScanRecordList* FTargets;
	TScanRecordList* FDuplicate;
	TScanRecordList* FUnique;
	TScanRecordList* FError;
	TScanOptions FOptions;
	int FLastProgress;
	void* FSyncronizeData;
	TMessageEvent FOnMessage;
	TProgressEvent FOnProgress;
	TResultsEvent FOnResults;
	TMessageEvent FOnError;
	void __fastcall Initialize(TScanRecordList* ATargets);
	void __fastcall Uninitialize();
	void __fastcall Scan();
	void __fastcall ListTargets();
	void __fastcall ListResults();
	void __fastcall WorkerContinue(TObject* ASender);
	void __fastcall ScanResultListerProgress(TObject* ASender, int ADone,
		int AAll);
	void __fastcall DoMessage(AnsiString AMessage);
	void __fastcall DoMessageInMainThread();
	void __fastcall DoProgress(int ADone, int AAll);
	void __fastcall DoProgressInMainThread();
	void __fastcall DoResults(TScanRecordList* ADuplicate,
		TScanRecordList* AUnique, TScanRecordList* AError);
	void __fastcall DoResultsInMainThread();
	void __fastcall DoError(AnsiString AMessage);
	void __fastcall DoErrorInMainThread();
	__property TScanRecordList* Targets = {read=FTargets};
	__property TScanRecordList* Duplicate = {read=FDuplicate};
	__property TScanRecordList* Unique = {read=FUnique};
	__property TScanRecordList* Error = {read=FError};
	__property TScanOptions Options = {read=FOptions};
	__property int LastProgress = {read=FLastProgress};
	__property void* SyncronizeData = {read=FSyncronizeData,
		write=FSyncronizeData};
protected:
	virtual void __fastcall Execute();
public:
	__fastcall TScanThread(TScanRecordList* ATargets, TScanOptions AOptions);
	virtual __fastcall ~TScanThread();
	__property TMessageEvent OnMessage = {read=FOnMessage, write=FOnMessage};
	__property TProgressEvent OnProgress = {read=FOnProgress,
		write=FOnProgress};
	__property TResultsEvent OnResults = {read=FOnResults, write=FOnResults};
	__property TMessageEvent OnError = {read=FOnError, write=FOnError};
};
//---------------------------------------------------------------------------
#endif

