//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "SystemUtils.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
// External
//---------------------------------------------------------------------------
// Combo Boxes
//---------------------------------------------------------------------------
void __fastcall UpdateComboBoxHistory(TComboBox* AComboBox,
	int AMaxItemCount)
{
	for(;;) {
		AnsiString text = AComboBox->Text;
		if(text.Length() == 0) {
			break;
		}
		for(int i = 0; i < AComboBox->Items->Count; i++) {
			if(AComboBox->Items->Strings[i] == text) {
				AComboBox->Items->Delete(i);
				break;
			}
		}
		while(AComboBox->Items->Count > 0 &&
				AComboBox->Items->Count >= AMaxItemCount) {
			AComboBox->Items->Delete(AComboBox->Items->Count - 1);
		}
		if(AMaxItemCount <= 0) {
			break;
		}
		AComboBox->Items->Insert(0, text);
		AComboBox->ItemIndex = 0;
		break;
	}
}
//---------------------------------------------------------------------------
// Files
//---------------------------------------------------------------------------
bool __fastcall FileOrDirExists(AnsiString AFileName)
{
	return GetFileAttributes(AFileName.c_str()) != 0xFFFFFFFF;
}
//---------------------------------------------------------------------------
bool __fastcall IsValidFileTime(const FILETIME& AFileTime)
{
	return !(AFileTime.dwLowDateTime == 0 && AFileTime.dwHighDateTime == 0);
}
//---------------------------------------------------------------------------
AnsiString __fastcall FullyQualifyFileName(AnsiString AFileName)
{
	AnsiString newFileName;
	for(;;) {
		AnsiString curFileName = AFileName;

		// Exapnd the filename
		if(curFileName.Length() == 0) {
			break;
		}
		curFileName = ExpandUNCFileName(curFileName);

		// Convert the filename to its long form
		{
			if(curFileName.LastDelimiter("*?") >= 1) {
				break;
			}
			if(curFileName.Length() >= MAX_PATH) {
				break;
			}
			TCHAR* buffer = NULL;
			try {
				DWORD bufferSize = 0;
				for(int i = 0; i < 2; i++) {
					DWORD stringLength = GetLongPathName(curFileName.c_str(),
						buffer, bufferSize);
					if(stringLength == 0) {
						goto LOOP_END;
					}
					if(i == 0) {
						bufferSize = stringLength;
						buffer = new TCHAR[bufferSize];
					}
					else {
						if(stringLength > bufferSize) {
							goto LOOP_END;
						}
						curFileName = AnsiString(buffer, stringLength);
					}
				}
			}
			__finally {
				delete[] buffer;
			}
		}

		newFileName = curFileName;
	LOOP_END:
		break;
	}
	return newFileName;
}
//---------------------------------------------------------------------------
// List Views
//---------------------------------------------------------------------------
AnsiString __fastcall ListColumnsToCommaText(TListColumns* AListColumns)
{
	AnsiString commaText;
	TStringList* descriptions = NULL;
	try {
		descriptions = new TStringList();
		for(int i = 0; i < AListColumns->Count; i++) {
			descriptions->Add(AListColumns->Items[i]->Caption);
		}
		commaText = descriptions->CommaText;
	}
	__finally {
		delete descriptions;
	}
	return commaText;
}
//---------------------------------------------------------------------------
AnsiString __fastcall ListItemToCommaText(TListItem* AListItem)
{
	AnsiString commaText;
	TStringList* descriptions = NULL;
	try {
		descriptions = new TStringList();
		descriptions->Add(AListItem->Caption);
		if(AListItem->SubItems->Count > 0) {
			descriptions->AddStrings(AListItem->SubItems);
		}
		commaText = descriptions->CommaText;
	}
	__finally {
		delete descriptions;
	}
	return commaText;
}
//---------------------------------------------------------------------------
// Numbers
//---------------------------------------------------------------------------
LARGE_INTEGER __fastcall MakeLargeInteger(DWORD ALowPart, LONG AHighPart)
{
	LARGE_INTEGER largeIntegerValue;
	largeIntegerValue.LowPart = ALowPart;
	largeIntegerValue.HighPart = AHighPart;
	return largeIntegerValue;
}
//---------------------------------------------------------------------------
LARGE_INTEGER __fastcall MakeLargeInteger(LONGLONG AQuadPart)
{
	LARGE_INTEGER largeIntegerValue;
	largeIntegerValue.QuadPart = AQuadPart;
	return largeIntegerValue;
}
//---------------------------------------------------------------------------
ULARGE_INTEGER __fastcall MakeULargeInteger(DWORD ALowPart, DWORD AHighPart)
{
	ULARGE_INTEGER uLargeIntegerValue;
	uLargeIntegerValue.LowPart = ALowPart;
	uLargeIntegerValue.HighPart = AHighPart;
	return uLargeIntegerValue;
}
//---------------------------------------------------------------------------
ULARGE_INTEGER __fastcall MakeULargeInteger(DWORDLONG AQuadPart)
{
	ULARGE_INTEGER uLargeIntegerValue;
	uLargeIntegerValue.QuadPart = AQuadPart;
	return uLargeIntegerValue;
}
//---------------------------------------------------------------------------
// Strings
//---------------------------------------------------------------------------
AnsiString __fastcall FormatInteger(AnsiString AIntegerString)
{
	AnsiString formattedString = AIntegerString;
	NUMBERFMT numberFormat;
	{
		ZeroMemory(&numberFormat, sizeof(NUMBERFMT));
		numberFormat.NumDigits = 0;
		numberFormat.LeadingZero = 1;
		numberFormat.Grouping = 3;
		numberFormat.lpDecimalSep = ".";
		numberFormat.lpThousandSep = ",";
		numberFormat.NegativeOrder = 1;
	}
	char* buffer = NULL;
	try {
		int bufferSize = 0;
		for(int i = 0; i < 2; i++) {
			::SetLastError(NO_ERROR);
			int stringLength = GetNumberFormat(
				LOCALE_USER_DEFAULT,
				0,
				AIntegerString.c_str(),
				&numberFormat,
				buffer,
				bufferSize);
			if(::GetLastError() != NO_ERROR) {
				break;
			}
			if(i == 0) {
				buffer = new char[stringLength];
				bufferSize = stringLength;
			}
			else {
				formattedString = AnsiString(buffer, stringLength);
			}
		}
	}
	__finally {
		delete[] buffer;
	}
	return formattedString;
}
//---------------------------------------------------------------------------
AnsiString __fastcall FormatFileTime(FILETIME AFileTime)
{
	AnsiString formattedString;
	for(;;) {
		SYSTEMTIME systemTime;
		if(!FileTimeToSystemTime(&AFileTime, &systemTime)) {
			break;
		}
		formattedString.sprintf("%u-%02u-%02u %02u:%02u:%02u",
			systemTime.wYear,
			systemTime.wMonth,
			systemTime.wDayOfWeek,
			systemTime.wDay,
			systemTime.wHour,
			systemTime.wMinute,
			systemTime.wSecond);
		break;
	}
	return formattedString;
}
//---------------------------------------------------------------------------

