//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "TClipboardViewer.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
// class TClipboardViewer : public TObject
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
void __fastcall TClipboardViewer::Initialize()
{
	try {
		InitializeWindow();
		InitializeChain();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TClipboardViewer::InitializeWindow()
{
	FHandle = AllocateHWnd(ProcessMessage);
}
//---------------------------------------------------------------------------
void __fastcall TClipboardViewer::InitializeChain()
{
	::SetLastError(NO_ERROR);
	FNextHandle = SetClipboardViewer(FHandle);
	if(::GetLastError() != NO_ERROR) {
		throw EOutOfResources(
			"Cannot add a window to the clipboard viewer chain");
	}
	FAdded = true;
}
//---------------------------------------------------------------------------
void __fastcall TClipboardViewer::Uninitialize()
{
	UninitializeChain();
	UninitializeWindow();
}
//---------------------------------------------------------------------------
void __fastcall TClipboardViewer::UninitializeWindow()
{
	if(Handle != NULL) {
		DeallocateHWnd(Handle);
	}
}
//---------------------------------------------------------------------------
void __fastcall TClipboardViewer::UninitializeChain()
{
	if(Added) {
		ChangeClipboardChain(Handle, NextHandle);
	}
}
//---------------------------------------------------------------------------
void __fastcall TClipboardViewer::ProcessMessage(TMessage& AMessage)
{
	if(AMessage.Msg == WM_CHANGECBCHAIN) {
		WMChangeCbChain(AMessage);
	}
	else if(AMessage.Msg == WM_DRAWCLIPBOARD) {
		WMDrawClipboard(AMessage);
	}
	else {
		AMessage.Result = DefWindowProc(FHandle, AMessage.Msg,
			AMessage.WParam, AMessage.LParam);
	}
}
//---------------------------------------------------------------------------
void __fastcall TClipboardViewer::WMChangeCbChain(TMessage& AMessage)
{
	if(reinterpret_cast<HWND>(AMessage.WParam) == NextHandle) {
		NextHandle = reinterpret_cast<HWND>(AMessage.LParam);
	}
	else {
		if(NextHandle != NULL) {
			SendMessage(NextHandle, AMessage.Msg, AMessage.WParam,
				AMessage.LParam);
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TClipboardViewer::WMDrawClipboard(TMessage& AMessage)
{
	if(OnChange != NULL) {
		OnChange(this);
	}
}
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TClipboardViewer::TClipboardViewer() :
	TObject(),
	FHandle(NULL),
	FNextHandle(NULL),
	FAdded(false),
	FOnChange(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TClipboardViewer::~TClipboardViewer()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
