//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "TFileFilter.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
// class EFileFilterError : public Exception
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall EFileFilterError::EFileFilterError(AnsiString AMessage) :
	Exception(AMessage)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall EFileFilterError::~EFileFilterError()
{
	// Do nothing
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// class TFileFilter
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TFileFilter::TFileFilter(bool AInclusive) :
	FInclusive(AInclusive)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TFileFilter::~TFileFilter()
{
	// Do nothing
}
//---------------------------------------------------------------------------
bool __fastcall TFileFilter::Passes(const WIN32_FIND_DATA& AFileData)
{
	bool passes = MatchFile(AFileData);
	if(!Inclusive) {
		passes = !passes;
	}
	return passes;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// class TFileExtensionFilter : public TFileFilter
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
int __fastcall TFileExtensionFilter::TFileNameList::CompareStrings(
	const AnsiString S1, const AnsiString S2)
{
	return AnsiCompareFileName(S1, S2);
}
//---------------------------------------------------------------------------
__fastcall TFileExtensionFilter::TFileNameList::TFileNameList() :
	TStringList()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TFileExtensionFilter::Initialize(TStrings* AFileExtensions)
{
	try {
		FFileExtensions = new TFileNameList();
		FileExtensions->Duplicates = dupIgnore;
		FileExtensions->Sorted = true;
		FileExtensions->AddStrings(AFileExtensions);
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileExtensionFilter::Uninitialize()
{
	delete FFileExtensions;
}
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
bool __fastcall TFileExtensionFilter::MatchFile(
	const WIN32_FIND_DATA& AFileData)
{
	int itemIndex;
	return FileExtensions->Find(ExtractFileExt(AFileData.cFileName),
		itemIndex);
}
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TFileExtensionFilter::TFileExtensionFilter(
		TStrings* AFileExtensions, bool AInclusive) :
	TFileFilter(AInclusive),
	FFileExtensions(NULL)
{
	Initialize(AFileExtensions);
}
//---------------------------------------------------------------------------
__fastcall TFileExtensionFilter::~TFileExtensionFilter()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TFileExtensionFilter::EnsurePair(TFileFilter* APartner)
{
	TFileExtensionFilter* partner =
		dynamic_cast<TFileExtensionFilter*>(APartner);
	if(partner->Inclusive != Inclusive) {
		for(int i = 0; i < FileExtensions->Count; i++) {
			int itemIndex;
			if(partner->FileExtensions->Find(FileExtensions->Strings[i],
					itemIndex)) {
				throw EFileFilterError("Ambiguous file extension filter");
			}
		}
	}
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// class TFileAttributeFilter : public TFileFilter
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
bool __fastcall TFileAttributeFilter::MatchFile(
	const WIN32_FIND_DATA& AFileData)
{
	return (AFileData.dwFileAttributes & FileAttributes) != 0;
}
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TFileAttributeFilter::TFileAttributeFilter(
		DWORD AFileAttributes, bool AInclusive) :
	TFileFilter(AInclusive),
	FFileAttributes(AFileAttributes)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TFileAttributeFilter::~TFileAttributeFilter()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TFileAttributeFilter::EnsurePair(TFileFilter* APartner)
{
	TFileAttributeFilter* partner =
		dynamic_cast<TFileAttributeFilter*>(APartner);
	if(partner->Inclusive != Inclusive) {
		if((partner->FileAttributes & FileAttributes) != 0) {
			throw EFileFilterError("Ambiguous file attribute filter");
		}
	}
}
//---------------------------------------------------------------------------

