//---------------------------------------------------------------------------
#ifndef TFileFilterH
#define TFileFilterH
#include <Classes.hpp>
//---------------------------------------------------------------------------
class EFileFilterError : public Exception
{
private:
protected:
public:
	__fastcall EFileFilterError(AnsiString AMessage);
	virtual __fastcall ~EFileFilterError();
};
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
class TFileFilter
{
private:
	bool FInclusive;
	__fastcall TFileFilter(const TFileFilter& ASrc);
	TFileFilter& __fastcall operator=(const TFileFilter& ASrc);
protected:
	virtual bool __fastcall MatchFile(const WIN32_FIND_DATA& AFileData) = 0;
public:
	__fastcall TFileFilter(bool AInclusive);
	virtual __fastcall ~TFileFilter();
	virtual bool __fastcall Passes(const WIN32_FIND_DATA& AFileData);
	virtual void __fastcall EnsurePair(TFileFilter* APartner) = 0;
	__property bool Inclusive = {read=FInclusive, write=FInclusive};
};
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
class TFileExtensionFilter : public TFileFilter
{
private:
	class TFileNameList : public TStringList
	{
	private:
		__property bool CaseSensitive = {};
	protected:
		virtual int __fastcall CompareStrings(const AnsiString S1,
			const AnsiString S2);
	public:
		__fastcall TFileNameList();
	};
	TFileNameList* FFileExtensions;
	void __fastcall Initialize(TStrings* AFileExtensions);
	void __fastcall Uninitialize();
	__property TFileNameList* FileExtensions = {read=FFileExtensions};
protected:
	virtual bool __fastcall MatchFile(const WIN32_FIND_DATA& AFileData);
public:
	__fastcall TFileExtensionFilter(TStrings* AFileExtensions,
		bool AInclusive);
	virtual __fastcall ~TFileExtensionFilter();
	virtual void __fastcall EnsurePair(TFileFilter* APartner);
};
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
class TFileAttributeFilter : public TFileFilter
{
private:
	DWORD FFileAttributes;
	__property DWORD FileAttributes = {read=FFileAttributes,
		write=FFileAttributes};
protected:
	virtual bool __fastcall MatchFile(const WIN32_FIND_DATA& AFileData);
public:
	__fastcall TFileAttributeFilter(DWORD AFileAttributes, bool AInclusive);
	virtual __fastcall ~TFileAttributeFilter();
	virtual void __fastcall EnsurePair(TFileFilter* APartner);
};
//---------------------------------------------------------------------------
#endif

