//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "TFileOperator.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
// class TFileOperator
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
void __fastcall TFileOperator::DoubleNullTerminateStrings(
	TMemoryStream* AStream, TStrings* AStrings) const
{
	// Get the total size
	int totalSize = 0;
	for(int i = 0; i < AStrings->Count; i++) {
		totalSize += AStrings->Strings[i].Length() + 1;
	}
	totalSize += 1;

	// Set strings
	AStream->Size = totalSize;
	AStream->Position = 0;
	for(int i = 0; i < AStrings->Count; i++) {
		AnsiString curString = AStrings->Strings[i];
		AStream->Write(static_cast<void*>(curString.c_str()),
			curString.Length() + 1);
	}
	AStream->Write(static_cast<void*>(AnsiString().c_str()), 1);
}
//---------------------------------------------------------------------------
bool __fastcall TFileOperator::OperateFiles(UINT AOperation,
	TStrings* ASrcFiles, TStrings* ADstFiles) const
{
	bool succeeded;
	TMemoryStream* srcFiles = NULL;
	TMemoryStream* dstFiles = NULL;
	try {
		srcFiles = new TMemoryStream();
		DoubleNullTerminateStrings(srcFiles, ASrcFiles);

		if(AOperation != FO_DELETE) {
			dstFiles = new TMemoryStream();
			DoubleNullTerminateStrings(dstFiles, ADstFiles);
		}

		SHFILEOPSTRUCT operationInfo;
		ZeroMemory(&operationInfo, sizeof(SHFILEOPSTRUCT));
		operationInfo.hwnd = Parent;
		operationInfo.wFunc = AOperation;
		operationInfo.pFrom = static_cast<char*>(srcFiles->Memory);
		operationInfo.pTo = NULL;
		if(AOperation != FO_DELETE) {
			operationInfo.pTo = static_cast<char*>(dstFiles->Memory);
		}
		operationInfo.fFlags = 0;
		if(AllowUndo) {
			operationInfo.fFlags |= FOF_ALLOWUNDO;
		}
		if(!ShowConfirmation) {
			operationInfo.fFlags |= FOF_NOCONFIRMATION;
		}
		if(!ConfirmMakeDir) {
			operationInfo.fFlags |= FOF_NOCONFIRMMKDIR;
		}
		if(AllowRename) {
			operationInfo.fFlags |= FOF_RENAMEONCOLLISION;
		}
		if(!ShowProgress) {
			operationInfo.fFlags |= FOF_SILENT;
		}
		if(AOperation != FO_DELETE && ADstFiles->Count > 1) {
			operationInfo.fFlags |= FOF_MULTIDESTFILES;
		}
		succeeded = SHFileOperation(&operationInfo) == 0;
	}
	__finally {
		delete dstFiles;
		delete srcFiles;
	}
	return succeeded;
}
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TFileOperator::TFileOperator() :
	FParent(NULL),
	FAllowUndo(false),
	FShowConfirmation(true),
	FConfirmMakeDir(true),
	FAllowRename(false),
	FShowProgress(true)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TFileOperator::~TFileOperator()
{
	// Do nothing
}
//---------------------------------------------------------------------------
bool __fastcall TFileOperator::CopyFiles(TStrings* ASrcFiles,
	TStrings* ADstFiles) const
{
	return OperateFiles(FO_COPY, ASrcFiles, ADstFiles);
}
//---------------------------------------------------------------------------
bool __fastcall TFileOperator::MoveFiles(TStrings* ASrcFiles,
	TStrings* ADstFiles) const
{
	return OperateFiles(FO_MOVE, ASrcFiles, ADstFiles);
}
//---------------------------------------------------------------------------
bool __fastcall TFileOperator::DeleteFiles(TStrings* AFiles) const
{
	return OperateFiles(FO_DELETE, AFiles, NULL);
}
//---------------------------------------------------------------------------
bool __fastcall TFileOperator::RenameFiles(TStrings* ASrcFiles,
	TStrings* ADstFiles) const
{
	return OperateFiles(FO_RENAME, ASrcFiles, ADstFiles);
}
//---------------------------------------------------------------------------

