//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "TShortCutFile.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
// class EShortCutFileError : public Exception
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall EShortCutFileError::EShortCutFileError(AnsiString AMessage) :
	Exception(AMessage)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall EShortCutFileError::~EShortCutFileError()
{
	// Do nothing
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// class TShortCutFile
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
void __fastcall TShortCutFile::Initialize()
{
	try {
		InitializeInterface();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TShortCutFile::InitializeInterface()
{
	bool succeeded = false;
	for(;;) {
		CoInitialize(NULL);
		if(FAILED(CoCreateInstance(CLSID_ShellLink, NULL,
				CLSCTX_INPROC_SERVER, IID_IShellLink,
				reinterpret_cast<void**>(&FShellLink)))) {
			break;
		}
		if(FAILED(ShellLink->QueryInterface(IID_IPersistFile,
				reinterpret_cast<void**>(&FPersistFile)))) {
			break;
		}
		succeeded = true;
		break;
	}
	if(!succeeded) {
		throw EShortCutFileError("Cannot initialize a shortcut file");
	}
}
//---------------------------------------------------------------------------
void __fastcall TShortCutFile::Uninitialize()
{
	UninitializeInterface();
}
//---------------------------------------------------------------------------
void __fastcall TShortCutFile::UninitializeInterface()
{
	if(FPersistFile != NULL) {
		FPersistFile->Release();
	}
	if(FShellLink != NULL) {
		FShellLink->Release();
	}
	CoUninitialize();
}
//---------------------------------------------------------------------------
AnsiString __fastcall TShortCutFile::GetArguments() const
{
	const int bufferSize = MAX_PATH;
	CHAR buffer[bufferSize];
	if(FAILED(ShellLink->GetArguments(buffer, bufferSize))) {
		ReadFail();
	}
	return AnsiString(buffer);
}
//---------------------------------------------------------------------------
void __fastcall TShortCutFile::SetArguments(AnsiString AArguments)
{
	if(FAILED(ShellLink->SetArguments(AArguments.c_str()))) {
		WriteFail();
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TShortCutFile::GetDescription() const
{
	const int bufferSize = MAX_PATH;
	CHAR buffer[bufferSize];
	if(FAILED(ShellLink->GetDescription(buffer, bufferSize))) {
		ReadFail();
	}
	return AnsiString(buffer);
}
//---------------------------------------------------------------------------
void __fastcall TShortCutFile::SetDescription(AnsiString ADescription)
{
	if(FAILED(ShellLink->SetDescription(ADescription.c_str()))) {
		WriteFail();
	}
}
//---------------------------------------------------------------------------
WORD __fastcall TShortCutFile::GetHotKey() const
{
	WORD hotKey;
	if(FAILED(ShellLink->GetHotkey(&hotKey))) {
		ReadFail();
	}
	return hotKey;
}
//---------------------------------------------------------------------------
void __fastcall TShortCutFile::SetHotKey(WORD AHotKey)
{
	if(FAILED(ShellLink->SetHotkey(AHotKey))) {
		WriteFail();
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TShortCutFile::GetIconFile() const
{
	const int bufferSize = MAX_PATH;
	CHAR buffer[bufferSize];
	int iconIndex;
	if(FAILED(ShellLink->GetIconLocation(buffer, bufferSize, &iconIndex))) {
		ReadFail();
	}
	return AnsiString(buffer);
}
//---------------------------------------------------------------------------
void __fastcall TShortCutFile::SetIconFile(AnsiString AIconFile)
{
	if(FAILED(ShellLink->SetIconLocation(AIconFile.c_str(), IconIndex))) {
		WriteFail();
	}
}
//---------------------------------------------------------------------------
int __fastcall TShortCutFile::GetIconIndex() const
{
	const int bufferSize = MAX_PATH;
	CHAR buffer[bufferSize];
	int iconIndex;
	if(FAILED(ShellLink->GetIconLocation(buffer, bufferSize, &iconIndex))) {
		ReadFail();
	}
	return iconIndex;
}
//---------------------------------------------------------------------------
void __fastcall TShortCutFile::SetIconIndex(int AIconIndex)
{
	if(FAILED(ShellLink->SetIconLocation(IconFile.c_str(), AIconIndex))) {
		WriteFail();
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TShortCutFile::GetTarget() const
{
	const int bufferSize = MAX_PATH;
	CHAR buffer[bufferSize];
	WIN32_FIND_DATA findData;
	if(FAILED(ShellLink->GetPath(buffer, bufferSize, &findData,
			SLGP_UNCPRIORITY))) {
		ReadFail();
	}
	return AnsiString(buffer);
}
//---------------------------------------------------------------------------
void __fastcall TShortCutFile::SetShowCommand(int AShowCommand)
{
	if(FAILED(ShellLink->SetShowCmd(AShowCommand))) {
		WriteFail();
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TShortCutFile::GetWorkingDirectory() const
{
	const int bufferSize = MAX_PATH;
	CHAR buffer[bufferSize];
	if(FAILED(ShellLink->GetWorkingDirectory(buffer, bufferSize))) {
		ReadFail();
	}
	return AnsiString(buffer);
}
//---------------------------------------------------------------------------
void __fastcall TShortCutFile::SetTarget(AnsiString ATarget)
{
	if(FAILED(ShellLink->SetPath(ATarget.c_str()))) {
		WriteFail();
	}
}
//---------------------------------------------------------------------------
int __fastcall TShortCutFile::GetShowCommand() const
{
	int showCommand;
	if(FAILED(ShellLink->GetShowCmd(&showCommand))) {
		ReadFail();
	}
	return showCommand;
}
//---------------------------------------------------------------------------
void __fastcall TShortCutFile::SetWorkingDirectory(
	AnsiString AWorkingDirectory)
{
	if(FAILED(ShellLink->SetWorkingDirectory(AWorkingDirectory.c_str()))) {
		WriteFail();
	}
}
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
void __fastcall TShortCutFile::Load(AnsiString AFileName)
{
	if(FAILED(PersistFile->Load(WideString(AFileName),
			STGM_READ | STGM_SHARE_DENY_WRITE))) {
		throw EShortCutFileError(AnsiString().sprintf(
			"Cannot load the shortcut file \"%s\"", AFileName.c_str()));
	}
}
//---------------------------------------------------------------------------
void __fastcall TShortCutFile::Save(AnsiString AFileName) const
{
	if(FAILED(PersistFile->Save(WideString(AFileName), true))) {
		throw EShortCutFileError(AnsiString().sprintf(
			"Cannot save the shortcut file \"%s\"", AFileName.c_str()));
	}
}
//---------------------------------------------------------------------------
void __fastcall TShortCutFile::ReadFail() const
{
	throw EShortCutFileError(AnsiString().sprintf(
		"Cannot read the shortcut file \"%s\"", FileName.c_str()));
}
//---------------------------------------------------------------------------
void __fastcall TShortCutFile::WriteFail() const
{
	throw EShortCutFileError(AnsiString().sprintf(
		"Cannot write the shortcut file \"%s\"", FileName.c_str()));
}
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TShortCutFile::TShortCutFile() :
	FFileName(),
	FShellLink(NULL),
	FPersistFile(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TShortCutFile::TShortCutFile(AnsiString AFileName) :
	FFileName(AFileName),
	FShellLink(NULL),
	FPersistFile(NULL)
{
	Initialize();
	try {
		Load(FileName);
	}
	catch(EShortCutFileError& e) {
		// Do nothing
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
__fastcall TShortCutFile::TShortCutFile(const TShortCutFile& ASrc) :
	FFileName(),
	FShellLink(NULL),
	FPersistFile(NULL)
{
	Initialize();
	try {
		*this = ASrc;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
__fastcall TShortCutFile::~TShortCutFile()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
TShortCutFile& __fastcall TShortCutFile::operator=(const TShortCutFile& ASrc)
{
	if(&ASrc != this) {
		FFileName = ASrc.FFileName;
		Arguments = ASrc.Arguments;
		Description = ASrc.Description;
		HotKey = ASrc.HotKey;
		IconFile = ASrc.IconFile;
		IconIndex = ASrc.IconIndex;
		ShowCommand = ASrc.ShowCommand;
		WorkingDirectory = ASrc.WorkingDirectory;
	}
	return *this;
}
//---------------------------------------------------------------------------
void __fastcall TShortCutFile::UpdateFile() const
{
	Save(FileName);
}
//---------------------------------------------------------------------------

