//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "TTextFindDialog.h"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TTextFindDialog* TextFindDialog = NULL;
//---------------------------------------------------------------------------
// class TFindNameDialog : public TForm
//---------------------------------------------------------------------------
// __published:
//---------------------------------------------------------------------------
void __fastcall TTextFindDialog::TextFindDialogShow(TObject* ASender)
{
	MakeBackup();
	Edit1->SelectAll();
	ActiveControl = Edit1;
}
//---------------------------------------------------------------------------
void __fastcall TTextFindDialog::TextFindDialogHide(TObject* ASender)
{
	if(!IsResultAccept(ModalResult)) {
		Reset();
	}
	FreeBackup();
}
//---------------------------------------------------------------------------
void __fastcall TTextFindDialog::Edit1Change(TObject* ASender)
{
	Change();
}
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
void __fastcall TTextFindDialog::MakeBackup()
{
	TBackup* backup = NULL;
	try {
		backup = new TBackup();
		backup->FindText = FindText;
		backup->Options = Options;
	}
	catch(...) {
		delete backup;
		throw;
	}
	FBackup = backup;
}
//---------------------------------------------------------------------------
void __fastcall TTextFindDialog::FreeBackup()
{
	delete FBackup;
	FBackup = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TTextFindDialog::Reset()
{
	FindText = Backup->FindText;
	Options = Backup->Options;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TTextFindDialog::GetFindText()
{
	AnsiString findText;
	if(Backup != NULL) {
		findText = Backup->FindText;
	}
	else {
		findText = Edit1->Text;
	}
	return findText;
}
//---------------------------------------------------------------------------
void __fastcall TTextFindDialog::SetFindText(AnsiString AFindText)
{
	if(Backup != NULL) {
		Backup->FindText = AFindText;
	}
	else {
		Edit1->Text = AFindText;
	}
	Change();
}
//---------------------------------------------------------------------------
TFindOptions __fastcall TTextFindDialog::GetOptions()
{
	TFindOptions options;
	if(Backup != NULL) {
		options = Backup->Options;
	}
	else {
		if(CheckBox1->Checked) {
			options << frMatchCase;
		}
		if(!CheckBox1->Enabled) {
			options << frDisableMatchCase;
		}
		if(!CheckBox1->Visible) {
			options << frHideMatchCase;
		}
		if(CheckBox2->Checked) {
			options << frWholeWord;
		}
		if(!CheckBox2->Enabled) {
			options << frDisableWholeWord;
		}
		if(!CheckBox2->Visible) {
			options << frHideWholeWord;
		}
		if(!RadioGroup1->Enabled) {
			options << frDisableUpDown;
		}
		if(RadioGroup1->ItemIndex != 0) {
			options << frDown;
		}
		if(!RadioGroup1->Visible) {
			options << frHideUpDown;
		}
	}
	return options;
}
//---------------------------------------------------------------------------
void __fastcall TTextFindDialog::SetOptions(TFindOptions AOptions)
{
	TFindOptions newOptions = AOptions;
	newOptions >> frFindNext;
	newOptions >> frReplace;
	newOptions >> frReplaceAll;
	newOptions >> frShowHelp;
	if(Backup != NULL) {
		Backup->Options = newOptions;
	}
	else {
		CheckBox1->Checked = Options.Contains(frMatchCase);
		CheckBox1->Enabled = !newOptions.Contains(frDisableMatchCase);
		CheckBox1->Visible = !newOptions.Contains(frHideMatchCase);
		CheckBox2->Checked = Options.Contains(frWholeWord);
		CheckBox2->Enabled = !newOptions.Contains(frDisableWholeWord);
		CheckBox2->Visible = !newOptions.Contains(frHideWholeWord);
		RadioGroup1->Enabled = !newOptions.Contains(frDisableUpDown);
		RadioGroup1->ItemIndex = Options.Contains(frDown) ? 1 : 0;
		RadioGroup1->Visible = !newOptions.Contains(frHideUpDown);
	}
	Change();
}
//---------------------------------------------------------------------------
void __fastcall TTextFindDialog::Change()
{
	Button1->Enabled = Edit1->Text.Length() > 0;
}
//---------------------------------------------------------------------------
bool __fastcall TTextFindDialog::IsResultAccept(TModalResult AResult) const
{
	return AResult == Button1->ModalResult;
}
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TTextFindDialog::TTextFindDialog(TComponent* AOwner) :
	TForm(AOwner),
	FBackup(NULL)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TTextFindDialog::~TTextFindDialog()
{
	if(Backup != NULL) {
		FreeBackup();
	}
}
//---------------------------------------------------------------------------
bool __fastcall TTextFindDialog::Execute()
{
	return IsResultAccept(ShowModal());
}
//---------------------------------------------------------------------------

