//---------------------------------------------------------------------------
#ifndef TTypedListH
#define TTypedListH
#include <Classes.hpp>
//---------------------------------------------------------------------------
template<class T>
class TTypedList : public TList
{
private:
	bool FOwnsObjects;
	T* __fastcall GetItem(int AIndex);
	void __fastcall SetItem(int AIndex, T* AItem);
protected:
	virtual void __fastcall Notify(void* APtr, TListNotification AAction);
public:
	__fastcall TTypedList();
	__fastcall TTypedList(bool AOwnsObjects);
	virtual __fastcall ~TTypedList();
	T* __fastcall operator[](int AIndex);
	int __fastcall Add(T* AItem);
	void __fastcall AddItems(TTypedList<T>* AList);
	void __fastcall AddExtractions(TTypedList<T>* ASrc);
	void __fastcall Insert(int AIndex, T* AItem);
	int __fastcall Remove(T* AItem);
	T* __fastcall Extract(T* AItem);
	T* __fastcall ExtractAt(int AIndex);
	void __fastcall ExtractAll();
	T* __fastcall ReplaceAt(int AIndex, T* AItem);
	T* __fastcall First();
	T* __fastcall Last();
	int __fastcall IndexOf(T* AItem);
	__property T* Items[int AIndex] = {read=GetItem, write=SetItem};
	__property bool OwnsObjects = {read=FOwnsObjects, write=FOwnsObjects};
};
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// template<class T>
// class TTypedList : public TList
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
template<class T>
T* __fastcall TTypedList<T>::GetItem(int AIndex)
{
	return static_cast<T*>(TList::Items[AIndex]);
}
//---------------------------------------------------------------------------
template<class T>
void __fastcall TTypedList<T>::SetItem(int AIndex, T* AItem)
{
	TList::Items[AIndex] = AItem;
}
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
template<class T>
void __fastcall TTypedList<T>::Notify(void* APtr, TListNotification AAction)
{
	if(AAction == lnDeleted) {
		if(OwnsObjects) {
			delete static_cast<T*>(APtr);
		}
	}
}
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
template<class T>
__fastcall TTypedList<T>::TTypedList() :
	TList(),
	FOwnsObjects(true)
{
	// Do nothing
}
//---------------------------------------------------------------------------
template<class T>
__fastcall TTypedList<T>::TTypedList(bool AOwnsObjects) :
	TList(),
	FOwnsObjects(AOwnsObjects)
{
	// Do nothing
}
//---------------------------------------------------------------------------
template<class T>
__fastcall TTypedList<T>::~TTypedList()
{
	// Do nothing
}
//---------------------------------------------------------------------------
template<class T>
T* __fastcall TTypedList<T>::operator[](int AIndex)
{
	return static_cast<T*>(TList::[AIndex]);
}
//---------------------------------------------------------------------------
template<class T>
int __fastcall TTypedList<T>::Add(T* AItem)
{
	return TList::Add(AItem);
}
//---------------------------------------------------------------------------
template<class T>
void __fastcall TTypedList<T>::AddItems(TTypedList<T>* AList)
{
	if(Capacity < Count + AList->Count) {
		Capacity = Count + AList->Count;
	}
	for(int i = 0; i < AList->Count; i++) {
		Add(AList->Items[i]);
	}
}
//---------------------------------------------------------------------------
template<class T>
void __fastcall TTypedList<T>::AddExtractions(TTypedList<T>* ASrc)
{
	AddItems(ASrc);
	ASrc->ExtractAll();
}
//---------------------------------------------------------------------------
template<class T>
void __fastcall TTypedList<T>::Insert(int AIndex, T* AItem)
{
	TList::Insert(AIndex, AItem);
}
//---------------------------------------------------------------------------
template<class T>
int __fastcall TTypedList<T>::Remove(T* AItem)
{
	return TList::Remove(AItem);
}
//---------------------------------------------------------------------------
template<class T>
T* __fastcall TTypedList<T>::Extract(T* AItem)
{
	return TList::Extract(AItem);
}
//---------------------------------------------------------------------------
template<class T>
T* __fastcall TTypedList<T>::ExtractAt(int AIndex)
{
	T* item = Items[AIndex];
	bool oldOwnsObjects = OwnsObjects;
	OwnsObjects = false;
	try {
		TList::Delete(AIndex);
	}
	__finally {
		OwnsObjects = oldOwnsObjects;
	}
	return item;
}
//---------------------------------------------------------------------------
template<class T>
void __fastcall TTypedList<T>::ExtractAll()
{
	bool oldOwnsObjects = OwnsObjects;
	OwnsObjects = false;
	try {
		TList::Clear();
	}
	__finally {
		OwnsObjects = oldOwnsObjects;
	}
}
//---------------------------------------------------------------------------
template<class T>
T* __fastcall TTypedList<T>::ReplaceAt(int AIndex, T* AItem)
{
	T* item = Items[AIndex];
	bool oldOwnsObjects = OwnsObjects;
	OwnsObjects = false;
	try {
		TList::Items[AIndex] = AItem;
	}
	__finally {
		OwnsObjects = oldOwnsObjects;
	}
	return item;
}
//---------------------------------------------------------------------------
template<class T>
T* __fastcall TTypedList<T>::First()
{
	return static_cast<T*>(TList::First());
}
//---------------------------------------------------------------------------
template<class T>
T* __fastcall TTypedList<T>::Last()
{
	return static_cast<T*>(TList::Last());
}
//---------------------------------------------------------------------------
template<class T>
int __fastcall TTypedList<T>::IndexOf(T* AItem)
{
	return TList::IndexOf(AItem);
}
//---------------------------------------------------------------------------
#endif

