//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "TVirtualListParams.h"
#include "TVirtualListView.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
// class TVirtualListParams : public TObject
//---------------------------------------------------------------------------
// private:
//---------------------------------------------------------------------------
void __fastcall TVirtualListParams::Initialize()
{
	try {
		FList = new TList();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TVirtualListParams::Uninitialize()
{
	delete FList;
}
//---------------------------------------------------------------------------
void* __fastcall TVirtualListParams::GetItem(int AIndex) const
{
	return List->Items[AIndex];
}
//---------------------------------------------------------------------------
void __fastcall TVirtualListParams::SetItem(int AIndex, void* AItem)
{
	if(AItem != List->Items[AIndex]) {
		DoDelete(List->Items[AIndex]);
		List->Items[AIndex] = AItem;
		DoChange();
	}
}
//---------------------------------------------------------------------------
int __fastcall TVirtualListParams::GetCount() const
{
	return List->Count;
}
//---------------------------------------------------------------------------
// protected:
//---------------------------------------------------------------------------
void __fastcall TVirtualListParams::DoAdd(void* AItem)
{
	if(OnAdd != NULL) {
		OnAdd(this, AItem);
	}
}
//---------------------------------------------------------------------------
void __fastcall TVirtualListParams::DoDelete(void* AItem)
{
	if(OnDelete != NULL) {
		OnDelete(this, AItem);
	}
}
//---------------------------------------------------------------------------
void __fastcall TVirtualListParams::DoChange()
{
	if(OnChange != NULL) {
		OnChange(this);
	}
}
//---------------------------------------------------------------------------
TIntegerDynArray __fastcall TVirtualListParams::GetAllIndexes() const
{
	TIntegerDynArray allIndexes;
	allIndexes.Length = Count;
	for(int i = 0; i < Count; i++) {
		allIndexes[i] = i;
	}
	return allIndexes;
}
//---------------------------------------------------------------------------
TIntegerDynArray __fastcall TVirtualListParams::GetSelIndexes() const
{
	TIntegerDynArray selIndexes;
	selIndexes.Length = ListView->SelCount;
	int indexCount = 0;
	for(int i = 0; i < Count; i++) {
		if(ListView->Selected[i]) {
			selIndexes[indexCount] = i;
			indexCount++;
		}
	}
	return selIndexes;
}
//---------------------------------------------------------------------------
TIntegerDynArray __fastcall TVirtualListParams::InvertIndexes(
	TIntegerDynArray AIndexes) const
{
	TIntegerDynArray newIndexes;
	newIndexes.Length = AIndexes.Length;
	for(int i = 0; i < AIndexes.Length; i++) {
		newIndexes[i] = AIndexes[AIndexes.Length - 1 - i];
	}
	return newIndexes;
}
//---------------------------------------------------------------------------
TBooleanDynArray __fastcall TVirtualListParams::IndexesToTable(
	TIntegerDynArray AIndexes) const
{
	TBooleanDynArray table;
	table.Length = Count;
	for(int i = 0; i < table.Length; i++) {
		table[i] = false;
	}
	for(int i = 0; i < AIndexes.Length; i++) {
		table[AIndexes[i]] = true;
	}
	return table;
}
//---------------------------------------------------------------------------
// public:
//---------------------------------------------------------------------------
__fastcall TVirtualListParams::TVirtualListParams(
		TVirtualListView* AListView) :
	TObject(),
	FListView(AListView),
	FList(NULL),
	FOnAdd(NULL),
	FOnDelete(NULL),
	FOnChange(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TVirtualListParams::~TVirtualListParams()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
int __fastcall TVirtualListParams::Add(void* AItem)
{
	int itemIndex = Count;
	Insert(itemIndex, AItem);
	return itemIndex;
}
//---------------------------------------------------------------------------
void __fastcall TVirtualListParams::AddItems(TList* AItems)
{
	InsertItems(Count, AItems);
}
//---------------------------------------------------------------------------
void __fastcall TVirtualListParams::Insert(int AIndex, void* AItem)
{
	TList* items = NULL;
	try {
		items = new TList();
		items->Add(AItem);
		InsertItems(AIndex, items);
	}
	__finally {
		delete items;
	}
}
//---------------------------------------------------------------------------
int __fastcall TVirtualListParams::Remove(void* AItem)
{
	int itemIndex = Find(AItem);
	if(itemIndex >= 0) {
		Delete(itemIndex);
	}
	return itemIndex;
}
//---------------------------------------------------------------------------
void __fastcall TVirtualListParams::RemoveItems(TList* AItems)
{
	TIntegerDynArray indexes;
	indexes.Length = AItems->Count;
	int indexCount = 0;
	for(int i = 0; i < AItems->Count; i++) {
		int itemIndex = Find(AItems->Items[i]);
		if(itemIndex >= 0) {
			indexes[indexCount] = itemIndex;
			indexCount++;
		}
	}
	if(indexes.Length != indexCount) {
		indexes.Length = indexCount;
	}
	DeleteItems(indexes);
}
//---------------------------------------------------------------------------
void __fastcall TVirtualListParams::Delete(int AIndex)
{
	TIntegerDynArray indexes;
	indexes.Length = 1;
	indexes[0] = AIndex;
	DeleteItems(indexes);
}
//---------------------------------------------------------------------------
void __fastcall TVirtualListParams::Clear()
{
	DeleteItems(GetAllIndexes());
}
//---------------------------------------------------------------------------
void __fastcall TVirtualListParams::Sort(TSortCompare ACompare, void* AData)
{
	struct TSortItem
	{
		int Index;
		TVirtualListParams* ListParams;
		TSortCompare Compare;
		void* Data;
		static int __fastcall CompareSortItem(void* AItem1, void* AItem2)
		{
			TSortItem* item1 = static_cast<TSortItem*>(AItem1);
			TSortItem* item2 = static_cast<TSortItem*>(AItem2);
			int difference = item1->Compare(
				item1->ListParams->Items[item1->Index],
				item2->ListParams->Items[item2->Index],
				item1->Data);
			if(difference == 0) {
				difference = item1->Index - item2->Index;
			}
			return difference;
		}
	};

	TIntegerDynArray newOrder;
	TList* sortItems = NULL;
	try {
		sortItems = new TList();
		for(int i = 0; i < Count; i++) {
			TSortItem sortItem = {i, this, ACompare, AData};
			sortItems->Add(new TSortItem(sortItem));
		}
		sortItems->Sort(TSortItem::CompareSortItem);
		newOrder.Length = sortItems->Count;
		for(int i = 0; i < sortItems->Count; i++) {
			newOrder[i] =
				static_cast<TSortItem*>(sortItems->Items[i])->Index;
		}
	}
	__finally {
		for(int i = 0; i < sortItems->Count; i++) {
			delete static_cast<TSortItem*>(sortItems->Items[i]);
		}
		delete sortItems;
	}
	Reorder(newOrder);
}
//---------------------------------------------------------------------------
void __fastcall TVirtualListParams::Invert()
{
	Reorder(InvertIndexes(GetAllIndexes()));
}
//---------------------------------------------------------------------------
int __fastcall TVirtualListParams::Find(void* AItem) const
{
	return List->IndexOf(AItem);
}
//---------------------------------------------------------------------------

